@@ -26,6 +26,7 @@ typedef unsigned char BackgroundType;
 #define BACKGROUND_COMMAND    2  /**< Command to run. */
 #define BACKGROUND_STRETCH    3  /**< Stretched image. */
 #define BACKGROUND_TILE       4  /**< Tiled image. */
+#define BACKGROUND_SCALE      5  /**< Scaled image. */
 
 /** Structure to represent a background for one or more desktops. */
 typedef struct BackgroundNode {
@@ -75,6 +76,7 @@ void StartupBackgrounds(void)
          break;
       case BACKGROUND_STRETCH:
       case BACKGROUND_TILE:
+      case BACKGROUND_SCALE:
          LoadImageBackground(bp);
          break;
       default:
@@ -123,6 +125,14 @@ void DestroyBackgrounds(void)
 /** Set the background to use for the specified desktops. */
 void SetBackground(int desktop, const char *type, const char *value)
 {
+   static const StringMappingType mapping[] = {
+      { "command",   BACKGROUND_COMMAND   },
+      { "gradient",  BACKGROUND_GRADIENT  },
+      { "image",     BACKGROUND_STRETCH   },
+      { "scale",     BACKGROUND_SCALE     },
+      { "solid",     BACKGROUND_SOLID     },
+      { "tile",      BACKGROUND_TILE      }
+   };
 
    BackgroundType bgType;
    BackgroundNode *bp;
@@ -135,19 +145,16 @@ void SetBackground(int desktop, const char *type, const char *value)
    }
 
    /* Parse the background type. */
-   if(!type || !strcmp(type, "solid")) {
+   if(type == NULL) {
       bgType = BACKGROUND_SOLID;
-   } else if(!strcmp(type, "gradient")) {
-      bgType = BACKGROUND_GRADIENT;
-   } else if(!strcmp(type, "command")) {
-      bgType = BACKGROUND_COMMAND;
-   } else if(!strcmp(type, "image")) {
-      bgType = BACKGROUND_STRETCH;
-   } else if(!strcmp(type, "tile")) {
-      bgType = BACKGROUND_TILE;
    } else {
-      Warning(_("invalid background type: \"%s\""), type);
-      return;
+      const int x = FindValue(mapping, ARRAY_LENGTH(mapping), type);
+      if(x >= 0) {
+         bgType = x;
+      } else {
+         Warning(_("invalid background type: \"%s\""), type);
+         return;
+      }
    }
 
    /* Remove the existing background if this is a duplicate.
@@ -183,7 +190,7 @@ void LoadBackground(int desktop)
 {
 
    XSetWindowAttributes attr;
-   long attrValues;
+   unsigned long attrValues;
    BackgroundNode *bp;
 
    /* Determine the background to load. */
@@ -264,12 +271,13 @@ void LoadGradientBackground(BackgroundNode *bp)
 
    /* Create the background pixmap. */
    if(color1.pixel == color2.pixel) {
-      bp->pixmap = JXCreatePixmap(display, rootWindow, 1, 1, rootDepth);
+      bp->pixmap = JXCreatePixmap(display, rootWindow, 1, 1,
+                                  rootVisual.depth);
       JXSetForeground(display, rootGC, color1.pixel);
       JXDrawPoint(display, bp->pixmap, rootGC, 0, 0);
    } else {
       bp->pixmap = JXCreatePixmap(display, rootWindow, 1, rootHeight,
-                                  rootDepth);
+                                  rootVisual.depth);
       DrawHorizontalGradient(bp->pixmap, rootGC, color1.pixel,
                              color2.pixel, 0, 0, 1, rootHeight);
    }
@@ -285,7 +293,7 @@ void LoadImageBackground(BackgroundNode *bp)
 
    /* Load the icon. */
    ExpandPath(&bp->value);
-   ip = LoadNamedIcon(bp->value, 0);
+   ip = LoadNamedIcon(bp->value, 0, bp->type == BACKGROUND_SCALE);
    if(JUNLIKELY(!ip)) {
       bp->pixmap = None;
       Warning(_("background image not found: \"%s\""), bp->value);
@@ -294,8 +302,8 @@ void LoadImageBackground(BackgroundNode *bp)
 
    /* Determine the size of the background pixmap. */
    if(bp->type == BACKGROUND_TILE) {
-      width = ip->image->width;
-      height = ip->image->height;
+      width = ip->images->width;
+      height = ip->images->height;
    } else {
       width = rootWidth;
       height = rootHeight;
@@ -303,14 +311,14 @@ void LoadImageBackground(BackgroundNode *bp)
 
    /* Create the pixmap. */
    bp->pixmap = JXCreatePixmap(display, rootWindow,
-                               width, height, rootDepth);
+                               width, height, rootVisual.depth);
 
    /* Clear the pixmap in case it is too small. */
    JXSetForeground(display, rootGC, 0);
    JXFillRectangle(display, bp->pixmap, rootGC, 0, 0, width, height);
 
    /* Draw the icon on the background pixmap. */
-   PutIcon(ip, bp->pixmap, 0, 0, 0, width, height);
+   PutIcon(&rootVisual, ip, bp->pixmap, 0, 0, 0, width, height);
 
    /* We don't need the icon anymore. */
    DestroyIcon(ip);
