@@ -10,21 +10,15 @@
 #include "jwm.h"
 #include "button.h"
 #include "border.h"
-#include "font.h"
-#include "color.h"
 #include "main.h"
 #include "icon.h"
 #include "image.h"
-#include "gradient.h"
-
-static void GetScaledIconSize(IconNode *ip, int maxsize,
-                              int *width, int *height);
+#include "misc.h"
 
 /** Draw a button. */
 void DrawButton(ButtonNode *bp)
 {
 
-   long outlinePixel;
    ColorType fg;
    long bg1, bg2;
 
@@ -40,11 +34,11 @@ void DrawButton(ButtonNode *bp)
    Assert(bp);
 
    drawable = bp->drawable;
-   gc = bp->gc;
    x = bp->x;
    y = bp->y;
    width = bp->width;
    height = bp->height;
+   gc = JXCreateGC(display, drawable, 0, NULL);
 
    /* Determine the colors to use. */
    switch(bp->type) {
@@ -52,76 +46,78 @@ void DrawButton(ButtonNode *bp)
       fg = COLOR_MENU_FG;
       bg1 = colors[COLOR_MENU_BG];
       bg2 = colors[COLOR_MENU_BG];
-      outlinePixel = colors[COLOR_MENU_BG];
       break;
    case BUTTON_MENU_ACTIVE:
       fg = COLOR_MENU_ACTIVE_FG;
       bg1 = colors[COLOR_MENU_ACTIVE_BG1];
       bg2 = colors[COLOR_MENU_ACTIVE_BG2];
-      outlinePixel = colors[COLOR_MENU_ACTIVE_OL];
       break;
    case BUTTON_TRAY:
       fg = COLOR_TRAYBUTTON_FG;
       bg1 = colors[COLOR_TRAYBUTTON_BG1];
       bg2 = colors[COLOR_TRAYBUTTON_BG2];
-      outlinePixel = colors[COLOR_TRAYBUTTON_DOWN];
       break;
    case BUTTON_TRAY_ACTIVE:
       fg = COLOR_TRAYBUTTON_ACTIVE_FG;
       bg1 = colors[COLOR_TRAYBUTTON_ACTIVE_BG1];
       bg2 = colors[COLOR_TRAYBUTTON_ACTIVE_BG2];
-      outlinePixel = colors[COLOR_TRAYBUTTON_ACTIVE_UP];
       break;
    case BUTTON_TASK:
       fg = COLOR_TASK_FG;
       bg1 = colors[COLOR_TASK_BG1];
       bg2 = colors[COLOR_TASK_BG2];
-      outlinePixel = colors[COLOR_TASK_DOWN];
       break;
    case BUTTON_TASK_ACTIVE:
       fg = COLOR_TASK_ACTIVE_FG;
       bg1 = colors[COLOR_TASK_ACTIVE_BG1];
       bg2 = colors[COLOR_TASK_ACTIVE_BG2];
-      outlinePixel = colors[COLOR_TASK_ACTIVE_UP];
       break;
    case BUTTON_MENU:
    default:
       fg = COLOR_MENU_FG;
       bg1 = colors[COLOR_MENU_BG];
       bg2 = colors[COLOR_MENU_BG];
-      outlinePixel = colors[COLOR_MENU_DOWN];
       break;
    }
 
    /* Draw the background. */
-   if(bp->border) {
+   if(bp->fill) {
 
       /* Draw the button background. */
       JXSetForeground(display, gc, bg1);
       if(bg1 == bg2) {
          /* single color */
-         JXFillRectangle(display, drawable, gc,
-                         x + 1, y + 1, width - 1, height - 1);
+         JXFillRectangle(display, drawable, gc, x, y, width, height);
       } else {
          /* gradient */
          DrawHorizontalGradient(drawable, gc, bg1, bg2,
-                                x + 1, y + 1, width - 2, height - 1);
+                                x, y, width, height);
       }
 
-      /* Draw the outline. */
-      JXSetForeground(display, gc, outlinePixel);
-      DrawRoundedRectangle(drawable, gc, x, y, width, height, 3);
+   }
 
+   /* Draw the border. */
+   if(bp->border) {
+      JXSetForeground(display, gc, colors[fg]);
+      JXDrawRectangle(display, drawable, gc, x, y, width, height);
    }
 
    /* Determine the size of the icon (if any) to display. */
    iconWidth = 0;
    iconHeight = 0;
    if(bp->icon) {
-      if(width < height) {
-         GetScaledIconSize(bp->icon, width - 5, &iconWidth, &iconHeight);
+      if(bp->icon == &emptyIcon) {
+         iconWidth = Min(width - 4, height - 4);
+         iconHeight = iconWidth;
       } else {
-         GetScaledIconSize(bp->icon, height - 5, &iconWidth, &iconHeight);
+         const int ratio = (bp->icon->images->width << 16)
+                         / bp->icon->images->height;
+         iconHeight = height - 4;
+         iconWidth = (iconHeight * ratio) >> 16;
+         if(iconWidth > width - 4) {
+            iconWidth = width - 4;
+            iconHeight = (iconWidth << 16) / ratio;
+         }
       }
    }
 
@@ -131,12 +127,12 @@ void DrawButton(ButtonNode *bp)
    if(bp->text) {
       textWidth = GetStringWidth(bp->font, bp->text);
       textHeight = GetStringHeight(bp->font);
-      if(textWidth + iconWidth + 8 > width) {
-         textWidth = width - iconWidth - 8;
-         if(textWidth < 0) {
-            textWidth = 0;
-         }
+      if(iconWidth > 0 && textWidth + iconWidth + 6 > width) {
+         textWidth = width - iconWidth - 6;
+      } else if(iconWidth == 0 && textWidth + 4 > width) {
+         textWidth = width - 4;
       }
+      textWidth = textWidth < 0 ? 0 : textWidth;
    }
 
    /* Determine the offset of the text in the button. */
@@ -146,36 +142,39 @@ void DrawButton(ButtonNode *bp)
          xoffset = 0;
       }
    } else {
-      xoffset = 3;
+      xoffset = 2;
    }
 
    /* Display the icon. */
    if(bp->icon) {
       yoffset = (height - iconHeight + 1) / 2;
-      PutIcon(bp->icon, drawable, colors[fg],
+      PutIcon(bp->visual, bp->icon, drawable, colors[fg],
               x + xoffset, y + yoffset,
               iconWidth, iconHeight);
       xoffset += iconWidth + 2;
    }
 
    /* Display the label. */
-   if(bp->text && textWidth) {
+   if(textWidth > 0) {
       yoffset = (height - textHeight + 1) / 2;
-      RenderString(drawable, bp->font, fg, x + xoffset, y + yoffset,
+      RenderString(bp->visual, drawable, bp->font, fg,
+                   x + xoffset, y + yoffset,
                    textWidth, bp->text);
    }
 
+   JXFreeGC(display, gc);
+
 }
 
 /** Reset a button node with default values. */
-void ResetButton(ButtonNode *bp, Drawable d, GC g)
+void ResetButton(ButtonNode *bp, Drawable d, const VisualData *visual)
 {
 
    Assert(bp);
 
    bp->type = BUTTON_MENU;
+   bp->visual = visual;
    bp->drawable = d;
-   bp->gc = g;
    bp->font = FONT_TRAY;
    bp->alignment = ALIGN_LEFT;
    bp->x = 0;
@@ -184,45 +183,7 @@ void ResetButton(ButtonNode *bp, Drawable d, GC g)
    bp->height = 1;
    bp->icon = NULL;
    bp->text = NULL;
-   bp->border = 1;
+   bp->fill = 1;
+   bp->border = 0;
 
 }
-
-/** Get the scaled size of an icon for a button. */
-void GetScaledIconSize(IconNode *ip, int maxsize,
-                       int *width, int *height)
-{
-
-   int ratio;
-
-   Assert(ip);
-   Assert(width);
-   Assert(height);
-
-   if(ip == &emptyIcon) {
-      *width = maxsize;
-      *height = maxsize;
-      return;
-   }
-
-   Assert(ip->image->height > 0);
-
-   /* Fixed point with 16-bit fraction. */
-   ratio = (ip->image->width << 16) / ip->image->height;
-
-   if(ip->image->width > ip->image->height) {
-
-      /* Compute size wrt width */
-      *width = maxsize;
-      *height = (*width << 16) / ratio;
-
-   } else {
-
-      /* Compute size wrt height */
-      *height = maxsize;
-      *width = (*height * ratio) >> 16;
-
-   }
-
-}
-
