@@ -10,9 +10,7 @@
 #include "jwm.h"
 #include "client.h"
 #include "clientlist.h"
-#include "main.h"
 #include "icon.h"
-#include "hint.h"
 #include "group.h"
 #include "tray.h"
 #include "confirm.h"
@@ -22,7 +20,6 @@
 #include "screen.h"
 #include "pager.h"
 #include "color.h"
-#include "error.h"
 #include "place.h"
 #include "event.h"
 #include "settings.h"
@@ -35,6 +32,7 @@ unsigned int clientCount;
 
 static void LoadFocus(void);
 static void ReparentClient(ClientNode *np, char notOwner);
+static void RestackTransients(const ClientNode *np);
 static void MinimizeTransients(ClientNode *np, char lower);
 static void RestoreTransients(ClientNode *np, char raise);
 static void KillClientHandler(ClientNode *np);
@@ -76,8 +74,8 @@ void StartupClients(void)
 
    LoadFocus();
 
-   UpdateTaskBar();
-   UpdatePager();
+   RequireTaskUpdate();
+   RequirePagerUpdate();
 
 }
 
@@ -144,17 +142,16 @@ ClientNode *AddClientWindow(Window w, char alreadyMapped, char notOwner)
    np->window = w;
    np->owner = None;
    np->state.desktop = currentDesktop;
-   np->controller = NULL;
-   np->name = NULL;
-   np->colormaps = NULL;
 
    np->x = attr.x;
    np->y = attr.y;
    np->width = attr.width;
    np->height = attr.height;
+   np->visual.depth = attr.depth;
+   np->visual.visual = attr.visual;
    np->cmap = attr.colormap;
-   np->colormaps = NULL;
    np->state.status = STAT_NONE;
+   np->state.maxFlags = MAX_NONE;
    np->state.layer = LAYER_NORMAL;
    np->state.defaultLayer = LAYER_NORMAL;
 
@@ -170,9 +167,10 @@ ClientNode *AddClientWindow(Window w, char alreadyMapped, char notOwner)
       np->state.defaultLayer = LAYER_ABOVE;
    }
 
-   LoadIcon(np);
-
    ApplyGroups(np);
+   if(np->icon == NULL) {
+      LoadIcon(np);
+   }
 
    /* We now know the layer, so insert */
    np->prev = NULL;
@@ -188,7 +186,7 @@ ClientNode *AddClientWindow(Window w, char alreadyMapped, char notOwner)
    ReparentClient(np, notOwner);
    PlaceClient(np, alreadyMapped);
 
-   if(!(np->state.status & (STAT_FULLSCREEN | STAT_VMAX | STAT_HMAX))) {
+   if(!((np->state.status & STAT_FULLSCREEN) || np->state.maxFlags)) {
       int north, south, east, west;
       GetBorderSize(&np->state, &north, &south, &east, &west);
       JXMoveResizeWindow(display, np->parent, np->x - west, np->y - north,
@@ -206,12 +204,15 @@ ClientNode *AddClientWindow(Window w, char alreadyMapped, char notOwner)
       JXMapWindow(display, np->parent);
    }
 
+   clientCount += 1;
+
    if(!alreadyMapped) {
       RaiseClient(np);
    }
 
-   clientCount += 1;
-
+   if(np->state.status & STAT_OPACITY) {
+      SetOpacity(np, np->state.opacity, 1);
+   }
    if(np->state.status & STAT_STICKY) {
       SetCardinalAtom(np->window, ATOM_NET_WM_DESKTOP, ~0UL);
    } else {
@@ -231,11 +232,10 @@ ClientNode *AddClientWindow(Window w, char alreadyMapped, char notOwner)
    }
 
    /* Maximize the client if requested. */
-   if(np->state.status & (STAT_HMAX | STAT_VMAX)) {
-      const char hmax = (np->state.status & STAT_HMAX) ? 1 : 0;
-      const char vmax = (np->state.status & STAT_VMAX) ? 1 : 0;
-      np->state.status &= ~(STAT_HMAX | STAT_VMAX);
-      MaximizeClient(np, hmax, vmax);
+   if(np->state.maxFlags) {
+      const MaxFlags flags = np->state.maxFlags;
+      np->state.maxFlags = MAX_NONE;
+      MaximizeClient(np, flags);
    }
 
    if(np->state.status & STAT_URGENT) {
@@ -280,8 +280,8 @@ void MinimizeClient(ClientNode *np, char lower)
 {
    Assert(np);
    MinimizeTransients(np, lower);
-   RestackClients();
-   UpdateTaskBar();
+   RequireRestack();
+   RequireTaskUpdate();
 }
 
 /** Minimize all transients as well as the specified client. */
@@ -356,7 +356,7 @@ void ShadeClient(ClientNode *np)
 
    WriteState(np);
    ResetBorder(np);
-   UpdatePager();
+   RequirePagerUpdate();
 
 }
 
@@ -379,7 +379,7 @@ void UnshadeClient(ClientNode *np)
    WriteState(np);
    ResetBorder(np);
    RefocusClient();
-   UpdatePager();
+   RequirePagerUpdate();
 
 }
 
@@ -409,8 +409,8 @@ void SetClientWithdrawn(ClientNode *np)
    np->state.status &= ~STAT_SDESKTOP;
 
    WriteState(np);
-   UpdateTaskBar();
-   UpdatePager();
+   RequireTaskUpdate();
+   RequirePagerUpdate();
 
 }
 
@@ -439,7 +439,6 @@ void RestoreTransients(ClientNode *np, char raise)
    np->state.status &= ~STAT_MINIMIZED;
    np->state.status &= ~STAT_SDESKTOP;
 
-
    /* Restore transient windows. */
    for(x = 0; x < LAYER_COUNT; x++) {
       for(tp = nodes[x]; tp; tp = tp->next) {
@@ -460,14 +459,9 @@ void RestoreTransients(ClientNode *np, char raise)
 /** Restore a client window and its transients. */
 void RestoreClient(ClientNode *np, char raise)
 {
-
-   Assert(np);
-
    RestoreTransients(np, raise);
-
-   RestackClients();
-   UpdateTaskBar();
-
+   RequireRestack();
+   RequireTaskUpdate();
 }
 
 /** Set the client layer. This will affect transients. */
@@ -520,7 +514,7 @@ void SetClientLayer(ClientNode *np, unsigned int layer)
          }
       }
 
-      RestackClients();
+      RequireRestack();
 
    }
 
@@ -611,8 +605,8 @@ void SetClientDesktop(ClientNode *np, unsigned int desktop)
             }
          }
       }
-      UpdatePager();
-      UpdateTaskBar();
+      RequirePagerUpdate();
+      RequireTaskUpdate();
    }
 
 }
@@ -654,10 +648,13 @@ void ShowClient(ClientNode *np)
 }
 
 /** Maximize a client window. */
-void MaximizeClient(ClientNode *np, char horiz, char vert)
+void MaximizeClient(ClientNode *np, MaxFlags flags)
 {
 
-   Assert(np);
+    /* Return if we don't have a client. */
+    if(np == NULL) {
+        return;
+    }
 
    /* Don't allow maximization of full-screen clients. */
    if(np->state.status & STAT_FULLSCREEN) {
@@ -671,21 +668,26 @@ void MaximizeClient(ClientNode *np, char horiz, char vert)
       UnshadeClient(np);
    }
 
-   if(np->state.status & (STAT_HMAX | STAT_VMAX)) {
+   RaiseClient(np);
+   FocusClient(np);
+   if(np->state.maxFlags) {
+      /* Undo existing maximization. */
       np->x = np->oldx;
       np->y = np->oldy;
       np->width = np->oldWidth;
       np->height = np->oldHeight;
-      np->state.status &= ~(STAT_HMAX | STAT_VMAX);
-   } else {
-      PlaceMaximizedClient(np, horiz, vert);
+      np->state.maxFlags = MAX_NONE;
+   }
+   if(flags != MAX_NONE) {
+      /* Maximize if requested. */
+      PlaceMaximizedClient(np, flags);
    }
 
    WriteState(np);
    ResetBorder(np);
    DrawBorder(np);
    SendConfigureEvent(np);
-   UpdatePager();
+   RequirePagerUpdate();
 
 }
 
@@ -693,14 +695,20 @@ void MaximizeClient(ClientNode *np, char horiz, char vert)
 void MaximizeClientDefault(ClientNode *np)
 {
 
-   char hmax, vmax;
+   MaxFlags flags = MAX_NONE;
 
    Assert(np);
 
-   hmax = (np->state.border & BORDER_MAX_H) ? 1 : 0;
-   vmax = (np->state.border & BORDER_MAX_V) ? 1 : 0;
+   if(np->state.maxFlags == MAX_NONE) {
+      if(np->state.border & BORDER_MAX_H) {
+         flags |= MAX_HORIZ;
+      }
+      if(np->state.border & BORDER_MAX_V) {
+         flags |= MAX_VERT;
+      }
+   }
 
-   MaximizeClient(np, hmax, vmax);
+   MaximizeClient(np, flags);
 
 }
 
@@ -719,6 +727,9 @@ void SetClientFullScreen(ClientNode *np, char fullScreen)
    if(!fullScreen == !(np->state.status & STAT_FULLSCREEN)) {
       return;
    }
+   if(!(np->state.border & BORDER_FULLSCREEN)) {
+      return;
+   }
 
    if(np->state.status & STAT_SHADED) {
       UnshadeClient(np);
@@ -728,7 +739,7 @@ void SetClientFullScreen(ClientNode *np, char fullScreen)
 
       np->state.status |= STAT_FULLSCREEN;
 
-      if(!(np->state.status & (STAT_HMAX | STAT_VMAX))) {
+      if(!(np->state.maxFlags)) {
          np->oldx = np->x;
          np->oldy = np->y;
          np->oldWidth = np->width;
@@ -751,7 +762,6 @@ void SetClientFullScreen(ClientNode *np, char fullScreen)
       ResetBorder(np);
 
    } else {
-      char vmax, hmax;
 
       np->state.status &= ~STAT_FULLSCREEN;
 
@@ -762,10 +772,8 @@ void SetClientFullScreen(ClientNode *np, char fullScreen)
       ConstrainSize(np);
       ConstrainPosition(np);
 
-      hmax = (np->state.status & STAT_HMAX) ? 1 : 0;
-      vmax = (np->state.status & STAT_VMAX) ? 1 : 0;
-      if(vmax || hmax) {
-         PlaceMaximizedClient(np, hmax, vmax);
+      if(np->state.maxFlags != MAX_NONE) {
+         PlaceMaximizedClient(np, np->state.maxFlags);
       }
 
       ResetBorder(np);
@@ -782,7 +790,7 @@ void SetClientFullScreen(ClientNode *np, char fullScreen)
 
    WriteState(np);
    SendConfigureEvent(np);
-   RestackClients();
+   RequireRestack();
 
 }
 
@@ -795,21 +803,28 @@ void FocusClient(ClientNode *np)
    if(np->state.status & STAT_HIDDEN) {
       return;
    }
+   if(!(np->state.status & STAT_CANFOCUS)) {
+      return;
+   }
 
    if(activeClient != np || !(np->state.status & STAT_ACTIVE)) {
 
       if(activeClient) {
          activeClient->state.status &= ~STAT_ACTIVE;
-         SetOpacity(activeClient, settings.inactiveClientOpacity);
+         if(!(activeClient->state.status & STAT_OPACITY)) {
+            SetOpacity(activeClient, settings.inactiveClientOpacity, 0);
+         }
          DrawBorder(activeClient);
       }
       np->state.status |= STAT_ACTIVE;
       activeClient = np;
-      SetOpacity(np, settings.activeClientOpacity);
+      if(!(np->state.status & STAT_OPACITY)) {
+         SetOpacity(np, settings.activeClientOpacity, 0);
+      }
 
       DrawBorder(np);
-      UpdatePager();
-      UpdateTaskBar();
+      RequirePagerUpdate();
+      RequireTaskUpdate();
 
    }
 
@@ -819,7 +834,7 @@ void FocusClient(ClientNode *np)
       if(np->state.status & STAT_CANFOCUS) {
          JXSetInputFocus(display, np->window, RevertToParent, eventTime);
       } else {
-         JXSetInputFocus(display, rootWindow, RevertToParent, eventTime);
+         JXSetInputFocus(display, np->parent, RevertToParent, eventTime);
       }
       if(np->state.status & STAT_TAKEFOCUS) {
          SendClientMessage(np->window, ATOM_WM_PROTOCOLS, ATOM_WM_TAKE_FOCUS);
@@ -880,18 +895,51 @@ void KillClient(ClientNode *np)
       NULL);
 }
 
+/** Place transients on top of the owner. */
+void RestackTransients(const ClientNode *np)
+{
+   ClientNode *tp;
+   unsigned int layer;
+
+   /* Place any transient windows on top of the owner */
+   for(layer = 0; layer < LAYER_COUNT; layer++) {
+      for(tp = nodes[layer]; tp; tp = tp->next) {
+         if(tp->owner == np->window && tp->prev) {
+
+            ClientNode *next = tp->next;
+
+            tp->prev->next = tp->next;
+            if(tp->next) {
+               tp->next->prev = tp->prev;
+            } else {
+               nodeTail[tp->state.layer] = tp->prev;
+            }
+            tp->next = nodes[tp->state.layer];
+            nodes[tp->state.layer]->prev = tp;
+            tp->prev = NULL;
+            nodes[tp->state.layer] = tp;
+
+            tp = next;
+
+         }
+
+         /* tp will be tp->next if the above code is executed. */
+         /* Thus, if it is NULL, we are done with this layer. */
+         if(!tp) {
+            break;
+         }
+      }
+   }
+}
+
 /** Raise the client. This will affect transients. */
 void RaiseClient(ClientNode *np)
 {
 
-   ClientNode *tp, *next;
-
    Assert(np);
 
    if(nodes[np->state.layer] != np) {
 
-      unsigned int x;
-
       /* Raise the window */
       Assert(np->prev);
       np->prev->next = np->next;
@@ -905,39 +953,117 @@ void RaiseClient(ClientNode *np)
       np->prev = NULL;
       nodes[np->state.layer] = np;
 
-      /* Place any transient windows on top of the owner */
-      for(x = 0; x < LAYER_COUNT; x++) {
-         for(tp = nodes[x]; tp; tp = tp->next) {
-            if(tp->owner == np->window && tp->prev) {
+   }
 
-               next = tp->next;
+   RestackTransients(np);
+   RequireRestack();
 
-               tp->prev->next = tp->next;
-               if(tp->next) {
-                  tp->next->prev = tp->prev;
-               } else {
-                  nodeTail[tp->state.layer] = tp->prev;
-               }
-               tp->next = nodes[tp->state.layer];
-               nodes[tp->state.layer]->prev = tp;
-               tp->prev = NULL;
-               nodes[tp->state.layer] = tp;
+}
 
-               tp = next;
+/** Restack a client window. This will not affect transients. */
+void RestackClient(ClientNode *np, Window above, int detail)
+{
 
-            }
+   ClientNode *tp;
+   char inserted = 0;
 
-            /* tp will be tp->next if the above code is executed. */
-            /* Thus, if it is NULL, we are done with this layer. */
-            if(!tp) {
+   /* Remove from the window list. */
+   if(np->prev) {
+      np->prev->next = np->next;
+   } else {
+      nodes[np->state.layer] = np->next;
+   }
+   if(np->next) {
+      np->next->prev = np->prev;
+   } else {
+      nodeTail[np->state.layer] = np->prev;
+   }
+
+   /* Insert back into the window list. */
+   if(above != None && above != np->window) {
+
+      /* Insert relative to some other window. */
+      char found = 0;
+      for(tp = nodes[np->state.layer]; tp; tp = tp->next) {
+         if(tp == np) {
+            found = 1;
+         } else if(tp->window == above) {
+            char insert_before = 0;
+            inserted = 1;
+            switch(detail) {
+            case Above:
+            case TopIf:
+               insert_before = 1;
+               break;
+            case Below:
+            case BottomIf:
+               insert_before = 0;
+               break;
+            case Opposite:
+               insert_before = !found;
                break;
             }
+            if(insert_before) {
+
+               /* Insert before this window. */
+               np->prev = tp->prev;
+               np->next = tp;
+               if(tp->prev) {
+                  tp->prev->next = np;
+               } else {
+                  nodes[np->state.layer] = np;
+               }
+               tp->prev = np;
+
+            } else {
+
+               /* Insert after this window. */
+               np->prev = tp;
+               np->next = tp->next;
+               if(tp->next) {
+                  tp->next->prev = np;
+               } else {
+                  nodeTail[np->state.layer] = np;
+               }
+               tp->next = np;
+
+            }
+            break;
          }
       }
-
-      RestackClients();
-
    }
+   if(!inserted) {
+
+      /* Insert absolute for the layer. */
+      if(detail == Below || detail == BottomIf) {
+
+         /* Insert to the bottom of the stack. */
+         np->next = NULL;
+         np->prev = nodeTail[np->state.layer];
+         if(nodeTail[np->state.layer]) {
+            nodeTail[np->state.layer]->next = np;
+         } else {
+            nodes[np->state.layer] = np;
+         }
+         nodeTail[np->state.layer] = np;
+
+      } else {
+
+         /* Insert at the top of the stack. */
+         np->next = nodes[np->state.layer];
+         np->prev = NULL;
+         if(nodes[np->state.layer]) {
+            nodes[np->state.layer]->prev = np;
+         } else {
+            nodeTail[np->state.layer] = np;
+         }
+         nodes[np->state.layer] = np;
+
+      }
+   }
+
+   RestackTransients(np);
+   RequireRestack();
 
 }
 
@@ -1006,7 +1132,7 @@ void RestackClients(void)
 
    ReleaseStack(stack);
    UpdateNetClientList();
-   UpdatePager();
+   RequirePagerUpdate();
 
 }
 
@@ -1077,7 +1203,7 @@ void RemoveClient(ClientNode *np)
    /* If the window manager is exiting (ie, not the client), then
     * reparent etc. */
    if(shouldExit && !(np->state.status & STAT_WMDIALOG)) {
-      if(np->state.status & (STAT_VMAX | STAT_HMAX)) {
+      if(np->state.maxFlags) {
          np->x = np->oldx;
          np->y = np->oldy;
          np->width = np->oldWidth;
@@ -1101,7 +1227,7 @@ void RemoveClient(ClientNode *np)
    }
 
    if(np->name) {
-      JXFree(np->name);
+      Release(np->name);
    }
    if(np->instanceName) {
       JXFree(np->instanceName);
@@ -1123,7 +1249,7 @@ void RemoveClient(ClientNode *np)
 
    Release(np);
 
-   RestackClients();
+   RequireRestack();
 
 }
 
@@ -1197,9 +1323,6 @@ void ReparentClient(ClientNode *np, char notOwner)
 
    attrMask = 0;
 
-   attrMask |= CWBackPixmap;
-   attr.background_pixmap = ParentRelative;
-
    /* We can't use PointerMotionHint mask here since the exact location
     * of the mouse on the frame is important. */
    attrMask |= CWEventMask;
@@ -1221,6 +1344,12 @@ void ReparentClient(ClientNode *np, char notOwner)
    attrMask |= CWBackPixel;
    attr.background_pixel = colors[COLOR_TITLE_BG2];
 
+   attrMask |= CWBorderPixel;
+   attr.border_pixel = 0;
+
+   attrMask |= CWColormap;
+   attr.colormap = np->cmap;
+
    x = np->x;
    y = np->y;
    width = np->width;
@@ -1233,8 +1362,8 @@ void ReparentClient(ClientNode *np, char notOwner)
 
    /* Create the frame window. */
    np->parent = JXCreateWindow(display, rootWindow, x, y, width, height,
-                               0, rootDepth, InputOutput,
-                               rootVisual, attrMask, &attr);
+                               0, np->visual.depth, InputOutput,
+                               np->visual.visual, attrMask, &attr);
  
    /* Update the window to get only the events we want. */
    attrMask = CWDontPropagate;
@@ -1245,6 +1374,13 @@ void ReparentClient(ClientNode *np, char notOwner)
       | ButtonMotionMask
       | KeyPressMask
       | KeyReleaseMask;
+
+   /* Make sure client doesn't muck with these. */
+   attrMask |= CWBackingStore;
+   attr.backing_store = NotUseful;
+   attrMask |= CWWinGravity;
+   attr.win_gravity = NorthWestGravity;
+
    JXChangeWindowAttributes(display, np->window, attrMask, &attr);
    JXSetWindowBorderWidth(display, np->window, 0);
 
@@ -1332,8 +1468,8 @@ void SignalUrgent(const TimeType *now, int x, int y, Window w, void *data)
       np->state.status |= STAT_FLASH;
    }
    DrawBorder(np);
-   UpdateTaskBar();
-   UpdatePager();
+   RequireTaskUpdate();
+   RequirePagerUpdate();
 
 }
 
