@@ -10,11 +10,9 @@
 #include "jwm.h"
 #include "confirm.h"
 #include "client.h"
-#include "main.h"
 #include "font.h"
 #include "button.h"
 #include "screen.h"
-#include "color.h"
 #include "misc.h"
 #include "settings.h"
 #include "key.h"
@@ -50,9 +48,6 @@ typedef struct {
 
 } DialogType;
 
-static const char *OK_STRING = "Ok";
-static const char *CANCEL_STRING = "Cancel";
-
 static DialogType *dialog = NULL;
 
 static int minWidth = 0;
@@ -68,6 +63,16 @@ static char HandleDialogButtonPress(const XButtonEvent *event);
 static char HandleDialogButtonRelease(const XButtonEvent *event);
 static char HandleDialogKeyPress(const XKeyEvent *event);
 
+static const char * const GetOKString()
+{
+   return _("OK");
+}
+
+static const char * const GetCancelString()
+{
+   return _("Cancel");
+}
+
 /** Stop dialog processing. */
 void ShutdownDialogs(void)
 {
@@ -281,7 +286,8 @@ void ShowConfirmDialog(ClientNode *np, void (*action)(ClientNode*), ...)
 
    /* Create the pixmap used for rendering. */
    dialog->pmap = JXCreatePixmap(display, rootWindow,
-                                 dialog->width, dialog->height, rootDepth);
+                                 dialog->width, dialog->height,
+                                 rootVisual.depth);
 
    /* Create the window. */
    attrs.background_pixel = colors[COLOR_MENU_BG];
@@ -298,7 +304,7 @@ void ShowConfirmDialog(ClientNode *np, void (*action)(ClientNode*), ...)
    shints.y = dialog->y;
    shints.flags = PPosition;
    JXSetWMNormalHints(display, window, &shints);
-   JXStoreName(display, window, "Confirm");
+   JXStoreName(display, window, _("Confirm"));
 
    /* Draw the dialog. */
    DrawDialog();
@@ -365,8 +371,8 @@ void ComputeDimensions(const ClientNode *np)
 
    /* Get the min width from the size of the buttons. */
    if(!minWidth) {
-      minWidth = GetStringWidth(FONT_MENU, CANCEL_STRING) * 3;
-      width = GetStringWidth(FONT_MENU, OK_STRING) * 3;
+      minWidth = GetStringWidth(FONT_MENU, GetCancelString()) * 3;
+      width = GetStringWidth(FONT_MENU, GetOKString()) * 3;
       if(width > minWidth) {
          minWidth = width;
       }
@@ -432,7 +438,7 @@ void DrawDialog(void)
    /* Draw the message. */
    yoffset = 4;
    for(x = 0; x < dialog->lineCount; x++) {
-      RenderString(dialog->pmap, FONT_MENU, COLOR_MENU_FG,
+      RenderString(&rootVisual, dialog->pmap, FONT_MENU, COLOR_MENU_FG,
                    4, yoffset, dialog->width, dialog->message[x]);
       yoffset += dialog->lineHeight;
    }
@@ -451,15 +457,16 @@ void DrawButtons(void)
 
    Assert(dialog);
 
-   dialog->buttonWidth = GetStringWidth(FONT_MENU, CANCEL_STRING);
-   temp = GetStringWidth(FONT_MENU, OK_STRING);
+   dialog->buttonWidth = GetStringWidth(FONT_MENU, GetCancelString());
+   temp = GetStringWidth(FONT_MENU, GetOKString());
    if(temp > dialog->buttonWidth) {
       dialog->buttonWidth = temp;
    }
    dialog->buttonWidth += 16;
    dialog->buttonHeight = dialog->lineHeight + 4;
 
-   ResetButton(&button, dialog->pmap, rootGC);
+   ResetButton(&button, dialog->pmap, &rootVisual);
+   button.border = 1;
    button.font = FONT_MENU;
    button.width = dialog->buttonWidth;
    button.height = dialog->buttonHeight;
@@ -475,7 +482,7 @@ void DrawButtons(void)
    } else {
       button.type = BUTTON_MENU;
    }
-   button.text = OK_STRING;
+   button.text = GetOKString();
    button.x = dialog->okx;
    button.y = dialog->buttony;
    DrawButton(&button);
@@ -485,7 +492,7 @@ void DrawButtons(void)
    } else {
       button.type = BUTTON_MENU;
    }
-   button.text = CANCEL_STRING;
+   button.text = GetCancelString();
    button.x = dialog->cancelx;
    button.y = dialog->buttony;
    DrawButton(&button);
