@@ -16,10 +16,8 @@
 #include "cursor.h"
 #include "desktop.h"
 #include "dock.h"
-#include "hint.h"
 #include "icon.h"
 #include "key.h"
-#include "main.h"
 #include "move.h"
 #include "place.h"
 #include "resize.h"
@@ -28,7 +26,6 @@
 #include "taskbar.h"
 #include "timing.h"
 #include "winmenu.h"
-#include "error.h"
 #include "settings.h"
 #include "tray.h"
 #include "popup.h"
@@ -49,6 +46,10 @@ typedef struct CallbackNode {
 
 static CallbackNode *callbacks = NULL;
 
+static char restack_pending = 0;
+static char task_update_pending = 0;
+static char pager_update_pending = 0;
+
 static void Signal(void);
 static void DispatchBorderButtonEvent(const XButtonEvent *event,
                                       ClientNode *np);
@@ -63,6 +64,7 @@ static char HandleDestroyNotify(const XDestroyWindowEvent *event);
 static void HandleMapRequest(const XMapEvent *event);
 static void HandleUnmapNotify(const XUnmapEvent *event);
 static void HandleButtonEvent(const XButtonEvent *event);
+static void ToggleMaximized(ClientNode *np, MaxFlags flags);
 static void HandleKeyPress(const XKeyEvent *event);
 static void HandleKeyRelease(const XKeyEvent *event);
 static void HandleEnterNotify(const XCrossingEvent *event);
@@ -71,10 +73,15 @@ static char HandleSelectionClear(const XSelectionClearEvent *event);
 
 static void HandleNetMoveResize(const XClientMessageEvent *event,
                                 ClientNode *np);
+static void HandleNetWMMoveResize(const XClientMessageEvent *evnet,
+                                  ClientNode *np);
+static void HandleNetRestack(const XClientMessageEvent *event,
+                             ClientNode *np);
 static void HandleNetWMState(const XClientMessageEvent *event,
                              ClientNode *np);
 static void HandleFrameExtentsRequest(const XClientMessageEvent *event);
 static void UpdateState(ClientNode *np);
+static void DiscardEnterEvents();
 
 #ifdef USE_SHAPE
 static void HandleShapeEvent(const XShapeEvent *event);
@@ -107,6 +114,17 @@ char WaitForEvent(XEvent *event)
 
    do {
 
+      if(restack_pending) {
+         RestackClients();
+         restack_pending = 0;
+      }
+      if(task_update_pending) {
+         UpdateTaskBar();
+      }
+      if(pager_update_pending) {
+         UpdatePager();
+      }
+
       while(JXPending(display) == 0) {
          FD_ZERO(&fds);
          FD_SET(fd, &fds);
@@ -311,9 +329,26 @@ void DiscardKeyEvents(XEvent *event, Window w)
    }
 }
 
+/** Discard enter notify events. */
+void DiscardEnterEvents()
+{
+   XEvent event;
+   JXSync(display, False);
+   while(JXCheckMaskEvent(display, EnterWindowMask, &event)) {
+      UpdateTime(&event);
+      SetMousePosition(event.xmotion.x_root, event.xmotion.y_root,
+                       event.xmotion.window);
+   }
+}
+
 /** Process a selection clear event. */
 char HandleSelectionClear(const XSelectionClearEvent *event)
 {
+   if(event->selection == managerSelection) {
+      /* Lost WM selection. */
+      shouldExit = 1;
+      return 1;
+   }
    return HandleDockSelectionClear(event);
 }
 
@@ -373,6 +408,18 @@ void HandleButtonEvent(const XButtonEvent *event)
 
 }
 
+/** Toggle maximized state. */
+void ToggleMaximized(ClientNode *np, MaxFlags flags)
+{
+   if(np) {
+      if(np->state.maxFlags == flags) {
+         MaximizeClient(np, MAX_NONE);
+      } else {
+         MaximizeClient(np, flags);
+      }
+   }
+}
+
 /** Process a key press event. */
 void HandleKeyPress(const XKeyEvent *event)
 {
@@ -461,9 +508,31 @@ void HandleKeyPress(const XKeyEvent *event)
       break;
    case KEY_MAX:
       if(np) {
-         MaximizeClient(np, 1, 1);
+         if(np->state.maxFlags) {
+            MaximizeClient(np, MAX_NONE);
+         } else {
+            MaximizeClient(np, MAX_HORIZ | MAX_VERT);
+         }
       }
       break;
+   case KEY_MAXTOP:
+      ToggleMaximized(np, MAX_TOP | MAX_HORIZ);
+      break;
+   case KEY_MAXBOTTOM:
+      ToggleMaximized(np, MAX_BOTTOM | MAX_HORIZ);
+      break;
+   case KEY_MAXLEFT:
+      ToggleMaximized(np, MAX_LEFT | MAX_VERT);
+      break;
+   case KEY_MAXRIGHT:
+      ToggleMaximized(np, MAX_RIGHT | MAX_VERT);
+      break;
+   case KEY_MAXV:
+      ToggleMaximized(np, MAX_VERT);
+      break;
+   case KEY_MAXH:
+      ToggleMaximized(np, MAX_HORIZ);
+      break;
    case KEY_ROOT:
       ShowKeyMenu(event);
       break;
@@ -487,9 +556,30 @@ void HandleKeyPress(const XKeyEvent *event)
          }
       }
       break;
+   case KEY_SENDR:
+      if(np) {
+         SetClientDesktop(np, GetRightDesktop(np->state.desktop));
+      }
+      break;
+   case KEY_SENDL:
+      if(np) {
+         SetClientDesktop(np, GetLeftDesktop(np->state.desktop));
+      }
+      break;
+   case KEY_SENDU:
+      if(np) {
+         SetClientDesktop(np, GetAboveDesktop(np->state.desktop));
+      }
+      break;
+   case KEY_SENDD:
+      if(np) {
+         SetClientDesktop(np, GetBelowDesktop(np->state.desktop));
+      }
+      break;
    default:
       break;
    }
+   DiscardEnterEvents();
 }
 
 /** Handle a key release event. */
@@ -518,9 +608,9 @@ void HandleConfigureRequest(const XConfigureRequestEvent *event)
    np = FindClientByWindow(event->window);
    if(np) {
 
+      int deltax, deltay;
       char changed = 0;
       char resized = 0;
-      int deltax, deltay;
 
       GetGravityDelta(np, np->gravity, &deltax, &deltay);
       if((event->value_mask & CWWidth) && (event->width != np->width)) {
@@ -580,6 +670,16 @@ void HandleConfigureRequest(const XConfigureRequestEvent *event)
          changed = 1;
       }
 
+      /* Update stacking. */
+      if((event->value_mask & CWStackMode)) {
+         Window above = None;
+         if(event->value_mask & CWSibling) {
+            above = event->above;
+         }
+         RestackClient(np, above, event->detail);
+      }
+
+      /* Return early if there's nothing to do. */
       if(!changed) {
          return;
       }
@@ -587,8 +687,8 @@ void HandleConfigureRequest(const XConfigureRequestEvent *event)
       if(np->controller) {
          (np->controller)(0);
       }
-      if(np->state.status & (STAT_VMAX | STAT_HMAX)) {
-         MaximizeClient(np, 0, 0);
+      if(np->state.maxFlags) {
+         MaximizeClient(np, MAX_NONE);
       }
 
       if(np->state.border & BORDER_CONSTRAIN) {
@@ -603,8 +703,9 @@ void HandleConfigureRequest(const XConfigureRequestEvent *event)
          GetBorderSize(&np->state, &north, &south, &east, &west);
          JXMoveWindow(display, np->parent, np->x - west, np->y - north);
       }
+
       SendConfigureEvent(np);
-      UpdatePager();
+      RequirePagerUpdate();
 
    } else {
 
@@ -742,14 +843,17 @@ char HandlePropertyNotify(const XPropertyEvent *event)
             UpdateState(np);
             WriteState(np);
             ResetBorder(np);
+         } else if(event->atom == atoms[ATOM_NET_WM_WINDOW_OPACITY]) {
+            ReadWMOpacity(np);
+            SetOpacity(np, np->state.opacity, 1);
          }
          break;
       }
 
       if(changed) {
          DrawBorder(np);
-         UpdateTaskBar();
-         UpdatePager();
+         RequireTaskUpdate();
+         RequirePagerUpdate();
       }
       if(np->state.status & STAT_WMDIALOG) {
          return 0;
@@ -823,6 +927,14 @@ void HandleClientMessage(const XClientMessageEvent *event)
 
          HandleNetMoveResize(event, np);
 
+      } else if(event->message_type == atoms[ATOM_NET_WM_MOVERESIZE]) {
+
+         HandleNetWMMoveResize(event, np);
+
+      } else if(event->message_type == atoms[ATOM_NET_RESTACK_WINDOW]) {
+
+         HandleNetRestack(event, np);
+
       } else if(event->message_type == atoms[ATOM_NET_WM_STATE]) {
 
          HandleNetWMState(event, np);
@@ -951,24 +1063,84 @@ void HandleNetMoveResize(const XClientMessageEvent *event, ClientNode *np)
    if(JUNLIKELY(np->state.status & STAT_FULLSCREEN)) {
       SetClientFullScreen(np, 0);
    }
-   if(JUNLIKELY(np->state.status & (STAT_HMAX | STAT_VMAX))) {
-      MaximizeClient(np, 0, 0);
+   if(JUNLIKELY(np->state.maxFlags)) {
+      MaximizeClient(np, MAX_NONE);
    }
 
    ConstrainSize(np);
    ResetBorder(np);
    SendConfigureEvent(np);
-   UpdatePager();
+   RequirePagerUpdate();
 
 }
 
+/** Handle a _NET_WM_MOVERESIZE request. */
+void HandleNetWMMoveResize(const XClientMessageEvent *event, ClientNode *np)
+{
+
+   const long x = event->data.l[0] - np->x;
+   const long y = event->data.l[1] - np->y;
+   const long direction = event->data.l[2];
+
+   switch(direction) {
+   case 0:  /* top-left */
+      ResizeClient(np, BA_RESIZE | BA_RESIZE_N | BA_RESIZE_W, x, y);
+      break;
+   case 1:  /* top */
+      ResizeClient(np, BA_RESIZE | BA_RESIZE_N, x, y);
+      break;
+   case 2:  /* top-right */
+      ResizeClient(np, BA_RESIZE | BA_RESIZE_N | BA_RESIZE_E, x, y);
+      break;
+   case 3:  /* right */
+      ResizeClient(np, BA_RESIZE | BA_RESIZE_E, x, y);
+      break;
+   case 4:  /* bottom-right */
+      ResizeClient(np, BA_RESIZE | BA_RESIZE_S | BA_RESIZE_E, x, y);
+      break;
+   case 5:  /* bottom */
+      ResizeClient(np, BA_RESIZE | BA_RESIZE_S, x, y);
+      break;
+   case 6:  /* bottom-left */
+      ResizeClient(np, BA_RESIZE | BA_RESIZE_S | BA_RESIZE_W, x, y);
+      break;
+   case 7:  /* left */
+      ResizeClient(np, BA_RESIZE | BA_RESIZE_W, x, y);
+      break;
+   case 8:  /* move */
+      MoveClient(np, x, y);
+      break;
+   case 9:  /* resize-keyboard */
+      ResizeClientKeyboard(np);
+      break;
+   case 10: /* move-keyboard */
+      MoveClientKeyboard(np);
+      break;
+   case 11: /* cancel */
+      if(np->controller) {
+         (np->controller)(0);
+      }
+      break;
+   default:
+      break;
+   }
+
+}
+
+/** Handle a _NET_RESTACK_WINDOW request. */
+void HandleNetRestack(const XClientMessageEvent *event, ClientNode *np)
+{
+   const Window sibling = event->data.l[1];
+   const int detail = event->data.l[2];
+   RestackClient(np, sibling, detail);
+}
+
 /** Handle a _NET_WM_STATE request. */
 void HandleNetWMState(const XClientMessageEvent *event, ClientNode *np)
 {
 
    unsigned int x;
-   char actionMaxH;
-   char actionMaxV;
+   MaxFlags maxFlags;
    char actionStick;
    char actionShade;
    char actionFullScreen;
@@ -979,8 +1151,7 @@ void HandleNetWMState(const XClientMessageEvent *event, ClientNode *np)
    char actionAbove;
 
    /* Up to two actions to be applied together. */
-   actionMaxH = 0;
-   actionMaxV = 0;
+   maxFlags = MAX_NONE;
    actionStick = 0;
    actionShade = 0;
    actionFullScreen = 0;
@@ -996,10 +1167,10 @@ void HandleNetWMState(const XClientMessageEvent *event, ClientNode *np)
          actionStick = 1;
       } else if(event->data.l[x]
          == (long)atoms[ATOM_NET_WM_STATE_MAXIMIZED_VERT]) {
-         actionMaxV = 1;
+         maxFlags |= MAX_VERT;
       } else if(event->data.l[x]
          == (long)atoms[ATOM_NET_WM_STATE_MAXIMIZED_HORZ]) {
-         actionMaxH = 1;
+         maxFlags |= MAX_HORIZ;
       } else if(event->data.l[x]
          == (long)atoms[ATOM_NET_WM_STATE_SHADED]) {
          actionShade = 1;
@@ -1029,10 +1200,8 @@ void HandleNetWMState(const XClientMessageEvent *event, ClientNode *np)
       if(actionStick) {
          SetClientSticky(np, 0);
       }
-      if(actionMaxH || actionMaxV) {
-         if(np->state.status & (STAT_HMAX | STAT_VMAX)) {
-            MaximizeClient(np, 0, 0);
-         }
+      if(maxFlags != MAX_NONE && np->state.maxFlags) {
+         MaximizeClient(np, np->state.maxFlags & ~maxFlags);
       }
       if(actionShade) {
          UnshadeClient(np);
@@ -1045,11 +1214,11 @@ void HandleNetWMState(const XClientMessageEvent *event, ClientNode *np)
       }
       if(actionNolist) {
          np->state.status &= ~STAT_NOLIST;
-         UpdateTaskBar();
+         RequireTaskUpdate();
       }
       if(actionNopager) {
          np->state.status &= ~STAT_NOPAGER;
-         UpdatePager();
+         RequirePagerUpdate();
       }
       if(actionBelow && np->state.layer == LAYER_BELOW) {
          SetClientLayer(np, np->state.defaultLayer);
@@ -1062,8 +1231,8 @@ void HandleNetWMState(const XClientMessageEvent *event, ClientNode *np)
       if(actionStick) {
          SetClientSticky(np, 1);
       }
-      if(!(np->state.status & (STAT_HMAX | STAT_VMAX))) {
-         MaximizeClient(np, actionMaxH, actionMaxV);
+      if(maxFlags != MAX_NONE) {
+         MaximizeClient(np, np->state.maxFlags | maxFlags);
       }
       if(actionShade) {
          ShadeClient(np);
@@ -1076,11 +1245,11 @@ void HandleNetWMState(const XClientMessageEvent *event, ClientNode *np)
       }
       if(actionNolist) {
          np->state.status |= STAT_NOLIST;
-         UpdateTaskBar();
+         RequireTaskUpdate();
       }
       if(actionNopager) {
          np->state.status |= STAT_NOPAGER;
-         UpdatePager();
+         RequirePagerUpdate();
       }
       if(actionBelow) {
          SetClientLayer(np, LAYER_BELOW);
@@ -1097,8 +1266,8 @@ void HandleNetWMState(const XClientMessageEvent *event, ClientNode *np)
             SetClientSticky(np, 1);
          }
       }
-      if(actionMaxH || actionMaxV) {
-         MaximizeClient(np, actionMaxH, actionMaxV);
+      if(maxFlags) {
+         MaximizeClient(np, np->state.maxFlags ^ maxFlags);
       }
       if(actionShade) {
          if(np->state.status & STAT_SHADED) {
@@ -1132,11 +1301,11 @@ void HandleNetWMState(const XClientMessageEvent *event, ClientNode *np)
        * recommendations. */
       if(actionNolist) {
          np->state.status ^= STAT_NOLIST;
-         UpdateTaskBar();
+         RequireTaskUpdate();
       }
       if(actionNopager) {
          np->state.status ^= STAT_NOPAGER;
-         UpdatePager();
+         RequirePagerUpdate();
       }
       break;
    default:
@@ -1245,11 +1414,11 @@ void HandleMapRequest(const XMapEvent *event)
             RaiseClient(np);
          }
          WriteState(np);
-         UpdateTaskBar();
-         UpdatePager();
+         RequireTaskUpdate();
+         RequirePagerUpdate();
       }
    }
-   RestackClients();
+   RequireRestack();
 }
 
 /** Handle an unmap notify event. */
@@ -1352,8 +1521,8 @@ void DispatchBorderButtonEvent(const XButtonEvent *event,
       if(event->button == Button1) {
          if(event->type == ButtonPress) {
             if(doubleClickActive
-               && abs(event->time - lastClickTime) > 0
-               && abs(event->time - lastClickTime) <= settings.doubleClickSpeed
+               && event->time != lastClickTime
+               && event->time - lastClickTime <= settings.doubleClickSpeed
                && abs(event->x - lastX) <= settings.doubleClickDelta
                && abs(event->y - lastY) <= settings.doubleClickDelta) {
                MaximizeClientDefault(np);
@@ -1401,10 +1570,10 @@ void DispatchBorderButtonEvent(const XButtonEvent *event,
             MaximizeClientDefault(np);
             break;
          case Button2:
-            MaximizeClient(np, 0, 1);
+            MaximizeClient(np, np->state.maxFlags ^ MAX_VERT);
             break;
          case Button3:
-            MaximizeClient(np, 1, 0);
+            MaximizeClient(np, np->state.maxFlags ^ MAX_HORIZ);
             break;
          default:
             break;
@@ -1427,6 +1596,7 @@ void DispatchBorderButtonEvent(const XButtonEvent *event,
    default:
       break;
    }
+   DiscardEnterEvents();
 }
 
 /** Update window state information. */
@@ -1547,3 +1717,20 @@ void UnregisterCallback(SignalCallback callback, void *data)
    Assert(0);
 }
 
+/** Restack clients before waiting for an event. */
+void RequireRestack()
+{
+   restack_pending = 1;
+}
+
+/** Update the task bar before waiting for an event. */
+void RequireTaskUpdate()
+{
+   task_update_pending = 1;
+}
+
+/** Update the pager before waiting for an event. */
+void RequirePagerUpdate()
+{
+   pager_update_pending = 1;
+}
