@@ -11,10 +11,9 @@
 #include "jwm.h"
 #include "hint.h"
 #include "client.h"
-#include "main.h"
-#include "tray.h"
 #include "desktop.h"
 #include "misc.h"
+#include "font.h"
 #include "settings.h"
 
 /* MWM Defines */
@@ -127,6 +126,7 @@ static const AtomNode atomList[] = {
    { &atoms[ATOM_NET_WM_ACTION_ABOVE],       "_NET_WM_ACTION_ABOVE"        },
    { &atoms[ATOM_NET_CLOSE_WINDOW],          "_NET_CLOSE_WINDOW"           },
    { &atoms[ATOM_NET_MOVERESIZE_WINDOW],     "_NET_MOVERESIZE_WINDOW"      },
+   { &atoms[ATOM_NET_RESTACK_WINDOW],        "_NET_RESTACK_WINDOW"         },
    { &atoms[ATOM_NET_REQUEST_FRAME_EXTENTS], "_NET_REQUEST_FRAME_EXTENTS"  },
    { &atoms[ATOM_NET_WM_PID],                "_NET_WM_PID"                 },
    { &atoms[ATOM_NET_WM_NAME],               "_NET_WM_NAME"                },
@@ -152,14 +152,23 @@ static const AtomNode atomList[] = {
    { &atoms[ATOM_NET_CLIENT_LIST_STACKING],  "_NET_CLIENT_LIST_STACKING"   },
    { &atoms[ATOM_NET_WM_STRUT_PARTIAL],      "_NET_WM_STRUT_PARTIAL"       },
    { &atoms[ATOM_NET_WM_STRUT],              "_NET_WM_STRUT"               },
-   { &atoms[ATOM_NET_SYSTEM_TRAY_OPCODE],    "_NET_SYSTEM_TRAY_OPCODE"     },
    { &atoms[ATOM_NET_WM_WINDOW_OPACITY],     &opacityAtom[0]               },
+   { &atoms[ATOM_NET_WM_MOVERESIZE],         "_NET_WM_MOVERESIZE"          },
+   { &atoms[ATOM_NET_SYSTEM_TRAY_OPCODE],    "_NET_SYSTEM_TRAY_OPCODE"     },
 
    { &atoms[ATOM_MOTIF_WM_HINTS],            "_MOTIF_WM_HINTS"             },
 
    { &atoms[ATOM_JWM_RESTART],               &jwmRestart[0]                },
    { &atoms[ATOM_JWM_EXIT],                  &jwmExit[0]                   },
-   { &atoms[ATOM_JWM_RELOAD],                &jwmReload[0]                 }
+   { &atoms[ATOM_JWM_RELOAD],                &jwmReload[0]                 },
+   { &atoms[ATOM_JWM_WM_STATE_MAXIMIZED_TOP],
+      "_JWM_WM_STATE_MAXIMIZED_TOP" },
+   { &atoms[ATOM_JWM_WM_STATE_MAXIMIZED_BOTTOM],
+      "_JWM_WM_STATE_MAXIMIZED_BOTTOM" },
+   { &atoms[ATOM_JWM_WM_STATE_MAXIMIZED_LEFT],
+      "_JWM_WM_STATE_MAXIMIZED_LEFT" },
+   { &atoms[ATOM_JWM_WM_STATE_MAXIMIZED_RIGHT],
+      "_JWM_WM_STATE_MAXIMIZED_RIGHT" }
 
 };
 
@@ -237,7 +246,7 @@ void StartupHints(void)
                     (unsigned char*)array, 2);
 
    /* _NET_WM_NAME */
-   win = GetSupportingWindow();
+   win = supportingWindow;
    JXChangeProperty(display, win, atoms[ATOM_NET_WM_NAME],
                     atoms[ATOM_UTF8_STRING], 8, PropModeReplace,
                     (unsigned char*)"JWM", 3);
@@ -298,6 +307,8 @@ void ReadClientInfo(ClientNode *np, char alreadyMapped)
    /* Read WM protocols. */
    ReadWMProtocols(np->window, &np->state);
 
+   ReadWMOpacity(np);
+
    /* Make sure this client is on at least as high of a layer
     * as its owner. */
    if(np->owner != None) {
@@ -342,17 +353,15 @@ void WriteState(ClientNode *np)
 }
 
 /** Set the opacity of a client. */
-void SetOpacity(struct ClientNode *np, unsigned int opacity)
+void SetOpacity(ClientNode *np, unsigned int opacity, char force)
 {
 
-   /* Just return if there's nothing to do. */
-   if(np->state.opacity == opacity) {
+   if(np->state.opacity == opacity && !force) {
       return;
    }
 
-   /* Update the opacity. */
    np->state.opacity = opacity;
-   if(opacity == UINT_MAX) {
+   if(opacity == 0xFFFFFFFF) {
       JXDeleteProperty(display, np->parent, atoms[ATOM_NET_WM_WINDOW_OPACITY]);
    } else {
       SetCardinalAtom(np->parent, ATOM_NET_WM_WINDOW_OPACITY, opacity);
@@ -363,7 +372,7 @@ void SetOpacity(struct ClientNode *np, unsigned int opacity)
 void WriteNetState(ClientNode *np)
 {
 
-   unsigned long values[8];
+   unsigned long values[16];
    int index;
 
    Assert(np);
@@ -376,12 +385,28 @@ void WriteNetState(ClientNode *np)
    } 
 
    index = 0;
-   if(np->state.status & STAT_HMAX) {
+   if(np->state.status & STAT_MINIMIZED) {
+      values[index++] = atoms[ATOM_NET_WM_STATE_HIDDEN];
+   }
+
+   if(np->state.maxFlags & MAX_HORIZ) {
       values[index++] = atoms[ATOM_NET_WM_STATE_MAXIMIZED_HORZ];
    }
-   if(np->state.status & STAT_VMAX) {
+   if(np->state.maxFlags & MAX_VERT) {
       values[index++] = atoms[ATOM_NET_WM_STATE_MAXIMIZED_VERT];
    }
+   if(np->state.maxFlags & MAX_TOP) {
+      values[index++] = atoms[ATOM_JWM_WM_STATE_MAXIMIZED_TOP];
+   }
+   if(np->state.maxFlags & MAX_BOTTOM) {
+      values[index++] = atoms[ATOM_JWM_WM_STATE_MAXIMIZED_BOTTOM];
+   }
+   if(np->state.maxFlags & MAX_LEFT) {
+      values[index++] = atoms[ATOM_JWM_WM_STATE_MAXIMIZED_LEFT];
+   }
+   if(np->state.maxFlags & MAX_RIGHT) {
+      values[index++] = atoms[ATOM_JWM_WM_STATE_MAXIMIZED_RIGHT];
+   }
 
    if(np->state.status & STAT_SHADED) {
       values[index++] = atoms[ATOM_NET_WM_STATE_SHADED];
@@ -533,6 +558,7 @@ ClientState ReadWindowState(Window win, char alreadyMapped)
    Assert(win != None);
 
    result.status = STAT_MAPPED;
+   result.maxFlags = MAX_NONE;
    result.border = BORDER_DEFAULT;
    result.layer = LAYER_NORMAL;
    result.defaultLayer = LAYER_NORMAL;
@@ -567,9 +593,17 @@ ClientState ReadWindowState(Window win, char alreadyMapped)
             } else if(state[x] == atoms[ATOM_NET_WM_STATE_SHADED]) {
                result.status |= STAT_SHADED;
             } else if(state[x] == atoms[ATOM_NET_WM_STATE_MAXIMIZED_VERT]) {
-               result.status |= STAT_VMAX;
+               result.maxFlags |= MAX_VERT;
             } else if(state[x] == atoms[ATOM_NET_WM_STATE_MAXIMIZED_HORZ]) {
-               result.status |= STAT_HMAX;
+               result.maxFlags |= MAX_HORIZ;
+            } else if(state[x] == atoms[ATOM_JWM_WM_STATE_MAXIMIZED_TOP]) {
+               result.maxFlags |= MAX_TOP;
+            } else if(state[x] == atoms[ATOM_JWM_WM_STATE_MAXIMIZED_BOTTOM]) {
+               result.maxFlags |= MAX_BOTTOM;
+            } else if(state[x] == atoms[ATOM_JWM_WM_STATE_MAXIMIZED_LEFT]) {
+               result.maxFlags |= MAX_LEFT;
+            } else if(state[x] == atoms[ATOM_JWM_WM_STATE_MAXIMIZED_RIGHT]) {
+               result.maxFlags |= MAX_RIGHT;
             } else if(state[x] == atoms[ATOM_NET_WM_STATE_FULLSCREEN]) {
                result.status |= STAT_FULLSCREEN;
             } else if(state[x] == atoms[ATOM_NET_WM_STATE_HIDDEN]) {
@@ -668,9 +702,6 @@ ClientState ReadWindowState(Window win, char alreadyMapped)
       result.status |= STAT_SHAPED;
    }
 
-   /* Note that since we set _NET_WM_WINDOW_OPACITY, we don't
-    * bother reading it. */
-
    return result;
 
 }
@@ -685,16 +716,9 @@ void ReadWMName(ClientNode *np)
    Atom realType;
    int realFormat;
    unsigned char *name;
-#ifdef USE_XUTF8
-   XTextProperty tprop;
-   char **text_list;
-   int tcount;
-#endif
-
-   Assert(np);
 
    if(np->name) {
-      JXFree(np->name);
+      Release(np->name);
    }
 
    status = JXGetWindowProperty(display, np->window,
@@ -704,7 +728,11 @@ void ReadWMName(ClientNode *np)
    if(status != Success || realFormat == 0) {
       np->name = NULL;
    } else {
-      np->name = (char*)name;
+      const size_t size = strlen((char*)name) + 1;
+      np->name = Allocate(size);
+      memcpy(np->name, name, size);
+      JXFree(name);
+      np->name = ConvertFromUTF8(np->name);
    }
 
 #ifdef USE_XUTF8
@@ -715,16 +743,19 @@ void ReadWMName(ClientNode *np)
                                    &realType, &realFormat, &count,
                                    &extra, &name);
       if(status == Success && realFormat == 8) {
+         char **tlist;
+         XTextProperty tprop;
+         int tcount;
          tprop.value = name;
          tprop.encoding = atoms[ATOM_COMPOUND_TEXT];
          tprop.format = realFormat;
-         tprop.nitems = strlen((char *)name);
-         if(Xutf8TextPropertyToTextList(display, &tprop, &text_list, &tcount)
+         tprop.nitems = strlen((char*)name);
+         if(XmbTextPropertyToTextList(display, &tprop, &tlist, &tcount)
             == Success && tcount > 0) {
-            const size_t len = strlen(text_list[0]) + 1;
-            np->name = Xmalloc(len);
-            memcpy(np->name, text_list[0], len);
-            XFreeStringList(text_list);
+            const size_t len = strlen(tlist[0]) + 1;
+            np->name = Allocate(len);
+            memcpy(np->name, tlist[0], len);
+            XFreeStringList(tlist);
          }
          JXFree(name);
       }
@@ -732,8 +763,12 @@ void ReadWMName(ClientNode *np)
 #endif
 
    if(!np->name) {
-      if(JXFetchName(display, np->window, &np->name) == 0) {
-         np->name = NULL;
+      char *temp = NULL;
+      if(JXFetchName(display, np->window, &temp)) {
+         const size_t len = strlen(temp) + 1;
+         np->name = Allocate(len);
+         memcpy(np->name, temp, len);
+         JXFree(temp);
       }
    }
 
@@ -966,6 +1001,17 @@ void ReadWMHints(Window win, ClientState *state, char alreadyMapped)
 
 }
 
+/** Read _NET_WM_WINDOW_OPACITY. */
+void ReadWMOpacity(ClientNode *np) {
+   unsigned long card;
+   if(GetCardinalAtom(np->window, ATOM_NET_WM_WINDOW_OPACITY, &card)) {
+      np->state.status |= STAT_OPACITY;
+   } else {
+      card = UINT_MAX;
+   }
+   np->state.opacity = (unsigned int)card;
+}
+
 /** Read _MOTIF_WM_HINTS */
 void ReadMotifHints(Window win, ClientState *state)
 {
