@@ -16,6 +16,7 @@
 #include "misc.h"
 #include "hint.h"
 #include "color.h"
+#include "settings.h"
 
 IconNode emptyIcon;
 
@@ -32,31 +33,34 @@ typedef struct IconPathNode {
    struct IconPathNode *next;
 } IconPathNode;
 
-static int iconSize = 0;
 static IconNode **iconHash;
 static IconPathNode *iconPaths;
 static IconPathNode *iconPathsTail;
 static GC iconGC;
+static char iconSizeSet = 0;
 
-static void SetIconSize(void);
 static void DoDestroyIcon(int index, IconNode *icon);
 static void ReadNetWMIcon(ClientNode *np);
+static void ReadWMHintIcon(ClientNode *np);
 static IconNode *CreateIcon(void);
 static IconNode *GetDefaultIcon(void);
 static IconNode *CreateIconFromData(const char *name, char **data);
-static IconNode *CreateIconFromFile(const char *fileName, char save);
+static IconNode *CreateIconFromDrawable(Drawable d, Pixmap mask);
+static IconNode *CreateIconFromFile(const char *fileName,
+                                    char save, char preserveAspect);
 static IconNode *CreateIconFromBinary(const unsigned long *data,
                                       unsigned int length);
 static IconNode *LoadNamedIconHelper(const char *name, const char *path,
-                                     char save);
+                                     char save, char preserveAspect);
 
 #if defined(USE_ICONS)
 static IconNode *LoadSuffixedIcon(const char *path, const char *name,
                                   const char *suffix);
 #endif
 
-static ScaledIconNode *GetScaledIcon(IconNode *icon, long fg,
-                                     int width, int height);
+static ImageNode *GetBestImage(IconNode *icon, int rwidth, int rheight);
+static ScaledIconNode *GetScaledIcon(IconNode *icon, ImageNode *iconImage,
+                                     long fg, int rwidth, int rheight);
 
 static void InsertIcon(IconNode *icon);
 static IconNode *FindIcon(const char *name);
@@ -75,16 +79,26 @@ void InitializeIcons(void)
       iconHash[x] = NULL;
    }
    memset(&emptyIcon, 0, sizeof(emptyIcon));
+   iconSizeSet = 0;
 }
 
 /** Startup icon support. */
 void StartupIcons(void)
 {
    XGCValues gcValues;
+   XIconSize iconSize;
    unsigned long gcMask;
    gcMask = GCGraphicsExposures;
    gcValues.graphics_exposures = False;
    iconGC = JXCreateGC(display, rootWindow, gcMask, &gcValues);
+
+   iconSize.min_width = settings.titleHeight - 4;
+   iconSize.min_height = settings.titleHeight - 4;
+   iconSize.max_width = iconSize.min_width;
+   iconSize.max_height = iconSize.min_height;
+   iconSize.width_inc = 1;
+   iconSize.height_inc = 1;
+   JXSetIconSizes(display, rootWindow, &iconSize, 1);
 }
 
 /** Shutdown icon support. */
@@ -116,30 +130,6 @@ void DestroyIcons(void)
    }
 }
 
-/** Set the preferred icon sizes on the root window. */
-void SetIconSize(void)
-{
-
-   XIconSize size;
-
-   if(!iconSize) {
-
-      /* FIXME: compute values based on the sizes we can actually use. */
-      iconSize = 32;
-
-      size.min_width = iconSize;
-      size.min_height = iconSize;
-      size.max_width = iconSize;
-      size.max_height = iconSize;
-      size.width_inc = iconSize;
-      size.height_inc = iconSize;
-
-      JXSetIconSizes(display, rootWindow, &size, 1);
-
-   }
-
-}
-
 /** Add an icon search path. */
 void AddIconPath(char *path)
 {
@@ -181,10 +171,10 @@ void AddIconPath(char *path)
 }
 
 /** Draw an icon. */
-void PutIcon(IconNode *icon, Drawable d, long fg,
+void PutIcon(const VisualData *visual, IconNode *icon, Drawable d, long fg,
              int x, int y, int width, int height)
 {
-
+   ImageNode *imageNode;
    ScaledIconNode *node;
 
    Assert(icon);
@@ -194,7 +184,8 @@ void PutIcon(IconNode *icon, Drawable d, long fg,
    }
 
    /* Scale the icon. */
-   node = GetScaledIcon(icon, fg, width, height);
+   imageNode = GetBestImage(icon, width, height);
+   node = GetScaledIcon(icon, imageNode, fg, width, height);
    if(node) {
 
       const int ix = x + (width - node->width) / 2;
@@ -203,7 +194,7 @@ void PutIcon(IconNode *icon, Drawable d, long fg,
       /* If we support xrender, use it. */
 #ifdef USE_XRENDER
       if(haveRender) {
-         PutScaledRenderIcon(icon, node, d, ix, iy);
+         PutScaledRenderIcon(visual, imageNode, d, ix, iy);
          return;
       }
 #endif
@@ -241,18 +232,22 @@ void LoadIcon(ClientNode *np)
 
    Assert(np);
 
-   SetIconSize();
-
    /* If client already has an icon, destroy it first. */
    DestroyIcon(np->icon);
    np->icon = NULL;
 
-   /* Attempt to read _NET_WM_ICON for an icon */
+   /* Attempt to read _NET_WM_ICON for an icon. */
    ReadNetWMIcon(np);
    if(np->icon) {
       return;
    }
 
+   /* Attempt to read an icon from XWMHints. */
+   ReadWMHintIcon(np);
+   if(np->icon) {
+      return;
+   }
+
    /* Attempt to find an icon for this program in the icon directory */
    if(np->instanceName) {
       for(ip = iconPaths; ip; ip = ip->next) {
@@ -324,7 +319,7 @@ IconNode *LoadSuffixedIcon(const char *path, const char *name,
    if(image) {
       result = CreateIcon();
       result->name = iconName;
-      result->image = image;
+      result->images = image;
       InsertIcon(result);
       return result;
    } else {
@@ -336,7 +331,7 @@ IconNode *LoadSuffixedIcon(const char *path, const char *name,
 #endif /* defined(USE_PNG) || defined(USE_XPM) || defined(USE_JPEG) */
 
 /** Load an icon from a file. */
-IconNode *LoadNamedIcon(const char *name, char save)
+IconNode *LoadNamedIcon(const char *name, char save, char preserveAspect)
 {
 
    IconPathNode *ip;
@@ -344,13 +339,11 @@ IconNode *LoadNamedIcon(const char *name, char save)
 
    Assert(name);
 
-   SetIconSize();
-
    if(name[0] == '/') {
-      return CreateIconFromFile(name, save);
+      return CreateIconFromFile(name, save, preserveAspect);
    } else {
       for(ip = iconPaths; ip; ip = ip->next) {
-         icon = LoadNamedIconHelper(name, ip->path, save);
+         icon = LoadNamedIconHelper(name, ip->path, save, preserveAspect);
          if(icon) {
             return icon;
          }
@@ -361,7 +354,8 @@ IconNode *LoadNamedIcon(const char *name, char save)
 }
 
 /** Helper for loading icons by name. */
-IconNode *LoadNamedIconHelper(const char *name, const char *path, char save)
+IconNode *LoadNamedIconHelper(const char *name, const char *path,
+                              char save, char preserveAspect)
 {
 
    IconNode *result;
@@ -370,7 +364,7 @@ IconNode *LoadNamedIconHelper(const char *name, const char *path, char save)
    temp = AllocateStack(strlen(name) + strlen(path) + 1);
    strcpy(temp, path);
    strcat(temp, name);
-   result = CreateIconFromFile(temp, save);
+   result = CreateIconFromFile(temp, save, preserveAspect);
    ReleaseStack(temp);
 
    return result;
@@ -380,6 +374,7 @@ IconNode *LoadNamedIconHelper(const char *name, const char *path, char save)
 /** Read the icon property from a client. */
 void ReadNetWMIcon(ClientNode *np)
 {
+   static const long MAX_LENGTH = 1 << 20;
    unsigned long count;
    int status;
    unsigned long extra;
@@ -387,7 +382,7 @@ void ReadNetWMIcon(ClientNode *np)
    int realFormat;
    unsigned char *data;
    status = JXGetWindowProperty(display, np->window, atoms[ATOM_NET_WM_ICON],
-                                0, 256 * 256 * 4, False, XA_CARDINAL,
+                                0, MAX_LENGTH, False, XA_CARDINAL,
                                 &realType, &realFormat, &count, &extra, &data);
    if(status == Success && realFormat != 0 && data) {
       np->icon = CreateIconFromBinary((unsigned long*)data, count);
@@ -395,6 +390,26 @@ void ReadNetWMIcon(ClientNode *np)
    }
 }
 
+/** Read the icon WMHint property from a client. */
+void ReadWMHintIcon(ClientNode *np)
+{
+   XWMHints *hints;
+   hints = JXGetWMHints(display, np->window);
+   if(hints) {
+      Drawable d = None;
+      Pixmap mask = None;
+      if(hints->flags & IconMaskHint) {
+         mask = hints->icon_mask;
+      }
+      if(hints->flags & IconPixmapHint) {
+         d = hints->icon_pixmap;
+      }
+      if(d != None) {
+         np->icon = CreateIconFromDrawable(d, mask);
+      }
+      JXFree(hints);
+   }
+}
 
 /** Create the default icon. */
 IconNode *GetDefaultIcon(void)
@@ -422,7 +437,7 @@ IconNode *CreateIconFromData(const char *name, char **data)
    if(image) {
       result = CreateIcon();
       result->name = CopyString(name);
-      result->image = image;
+      result->images = image;
       InsertIcon(result);
       return result;
    } else {
@@ -431,8 +446,23 @@ IconNode *CreateIconFromData(const char *name, char **data)
 
 }
 
+IconNode *CreateIconFromDrawable(Drawable d, Pixmap mask)
+{
+   ImageNode *image;
+
+   image = LoadImageFromDrawable(d, mask);
+   if(image) {
+      IconNode *result = CreateIcon();
+      result->images = image;
+      return result;
+   } else {
+      return NULL;
+   }
+}
+
 /** Create an icon from the specified file. */
-IconNode *CreateIconFromFile(const char *fileName, char save)
+IconNode *CreateIconFromFile(const char *fileName,
+                             char save, char preserveAspect)
 {
 
    ImageNode *image;
@@ -451,7 +481,8 @@ IconNode *CreateIconFromFile(const char *fileName, char save)
    image = LoadImage(fileName);
    if(image) {
       result = CreateIcon();
-      result->image = image;
+      result->preserveAspect = preserveAspect;
+      result->images = image;
       if(save) {
          result->name = CopyString(fileName);
          InsertIcon(result);
@@ -463,13 +494,56 @@ IconNode *CreateIconFromFile(const char *fileName, char save)
 
 }
 
+/** Get the best image for the requested size. */
+ImageNode *GetBestImage(IconNode *icon, int rwidth, int rheight)
+{
+   /* Find the best image to use.
+    * Select the smallest image to completely cover the
+    * requested size.  If no image completely covers the
+    * requested size, select the one that overlaps the most area.
+    * If no size is specified, use the largest. */
+   ImageNode *best = icon->images;
+   ImageNode *ip = icon->images->next;
+   while(ip) {
+      const int best_area = best->width * best->height;
+      const int other_area = ip->width * ip->height;
+      int best_overlap;
+      int other_overlap;
+      if(rwidth == 0 && rheight == 0) {
+         best_overlap = 0;
+         other_overlap = 0;
+      } else if(rwidth == 0) {
+         best_overlap = Min(best->height, rheight);
+         other_overlap = Min(ip->height, rheight);
+      } else if(rheight == 0) {
+         best_overlap = Min(best->width, rwidth);
+         other_overlap = Min(ip->width, rwidth);
+      } else {
+         best_overlap = Min(best->width, rwidth)
+                      * Min(best->height, rheight);
+         other_overlap = Min(ip->width, rwidth)
+                       * Min(ip->height, rheight);
+      }
+      if(other_overlap > best_overlap) {
+         best = ip;
+      } else if(other_overlap == best_overlap) {
+         if(other_area < best_area) {
+            best = ip;
+         }
+      }
+      ip = ip->next;
+   }
+   return best;
+}
+
 /** Get a scaled icon. */
-ScaledIconNode *GetScaledIcon(IconNode *icon, long fg,
-                              int rwidth, int rheight)
+ScaledIconNode *GetScaledIcon(IconNode *icon, ImageNode *iconImage,
+                              long fg, int rwidth, int rheight)
 {
 
    XColor color;
    XImage *image;
+   XPoint *points;
    ScaledIconNode *np;
    GC maskGC;
    int x, y;
@@ -479,31 +553,29 @@ ScaledIconNode *GetScaledIcon(IconNode *icon, long fg,
    int nwidth, nheight;
    unsigned char *data;
 
-   Assert(icon);
-   Assert(icon->image);
-
    if(rwidth == 0) {
-      rwidth = icon->image->width;
+      rwidth = iconImage->width;
    }
    if(rheight == 0) {
-      rheight = icon->image->height;
+      rheight = iconImage->height;
    }
 
-   ratio = (icon->image->width << 16) / icon->image->height;
-   nwidth = Min(rwidth, (rheight * ratio) >> 16);
-   nheight = Min(rheight, (nwidth << 16) / ratio);
-   nwidth = (nheight * ratio) >> 16;
-   if(nwidth < 1) {
-      nwidth = 1;
-   }
-   if(nheight < 1) {
-      nheight = 1;
+   if(icon->preserveAspect) {
+      ratio = (iconImage->width << 16) / iconImage->height;
+      nwidth = Min(rwidth, (rheight * ratio) >> 16);
+      nheight = Min(rheight, (nwidth << 16) / ratio);
+      nwidth = (nheight * ratio) >> 16;
+   } else {
+      nheight = rheight;
+      nwidth = rwidth;
    }
+   nwidth = Max(1, nwidth);
+   nheight = Max(1, nheight);
 
    /* Check if this size already exists.
     * Note that XRender scales on the fly.
     */
-   for(np = icon->nodes; np; np = np->next) {
+   for(np = iconImage->nodes; np; np = np->next) {
 #ifdef USE_XRENDER
       if(np->imagePicture != None) {
          np->width = nwidth;
@@ -512,7 +584,7 @@ ScaledIconNode *GetScaledIcon(IconNode *icon, long fg,
       }
 #endif
       if(np->width == nwidth && np->height == nheight) {
-         if(!icon->image->bitmap || np->fg == fg) {
+         if(!iconImage->bitmap || np->fg == fg) {
             return np;
          }
       }
@@ -521,11 +593,14 @@ ScaledIconNode *GetScaledIcon(IconNode *icon, long fg,
    /* See if we can use XRender to create the icon. */
 #ifdef USE_XRENDER
    if(haveRender) {
-      np = CreateScaledRenderIcon(icon, fg, nwidth, nheight);
+      np = CreateScaledRenderIcon(iconImage, fg);
 
       /* Don't keep the image data around after creating the icon. */
-      Release(icon->image->data);
-      icon->image->data = NULL;
+      Release(iconImage->data);
+      iconImage->data = NULL;
+
+      np->width = nwidth;
+      np->height = nheight;
 
       return np;
    }
@@ -536,11 +611,11 @@ ScaledIconNode *GetScaledIcon(IconNode *icon, long fg,
    np->fg = fg;
    np->width = nwidth;
    np->height = nheight;
-   np->next = icon->nodes;
+   np->next = iconImage->nodes;
 #ifdef USE_XRENDER
    np->imagePicture = None;
 #endif
-   icon->nodes = np;
+   iconImage->nodes = np;
 
    /* Create a mask. */
    np->mask = JXCreatePixmap(display, rootWindow, nwidth, nheight, 1);
@@ -550,30 +625,34 @@ ScaledIconNode *GetScaledIcon(IconNode *icon, long fg,
    JXSetForeground(display, maskGC, 1);
 
    /* Create a temporary XImage for scaling. */
-   image = JXCreateImage(display, rootVisual, rootDepth, ZPixmap, 0,
-                         NULL, nwidth, nheight, 8, 0);
+   image = JXCreateImage(display, rootVisual.visual, rootVisual.depth,
+                         ZPixmap, 0, NULL, nwidth, nheight, 8, 0);
    image->data = Allocate(sizeof(unsigned long) * nwidth * nheight);
 
    /* Determine the scale factor. */
-   scalex = (icon->image->width << 16) / nwidth;
-   scaley = (icon->image->height << 16) / nheight;
+   scalex = (iconImage->width << 16) / nwidth;
+   scaley = (iconImage->height << 16) / nheight;
 
-   data = icon->image->data;
+   points = Allocate(sizeof(XPoint) * nwidth);
+   data = iconImage->data;
    srcy = 0;
    for(y = 0; y < nheight; y++) {
-      const int yindex = (srcy >> 16) * icon->image->width;
+      const int yindex = (srcy >> 16) * iconImage->width;
+      int pindex = 0;
       srcx = 0;
       for(x = 0; x < nwidth; x++) {
-         if(icon->image->bitmap) {
+         if(iconImage->bitmap) {
             const int index = yindex + (srcx >> 16);
             const int offset = index >> 3;
             const int mask = 1 << (index & 7);
             if(data[offset] & mask) {
-               JXDrawPoint(display, np->mask, maskGC, x, y);
+               points[pindex].x = x;
+               points[pindex].y = y;
                XPutPixel(image, x, y, fg);
+               pindex += 1;
             }
          } else {
-            const int yindex = (srcy >> 16) * icon->image->width;
+            const int yindex = (srcy >> 16) * iconImage->width;
             const int index = 4 * (yindex + (srcx >> 16));
             color.red = data[index + 1];
             color.red |= color.red << 8;
@@ -584,19 +663,24 @@ ScaledIconNode *GetScaledIcon(IconNode *icon, long fg,
             GetColor(&color);
             XPutPixel(image, x, y, color.pixel);
             if(data[index] >= 128) {
-               JXDrawPoint(display, np->mask, maskGC, x, y);
+               points[pindex].x = x;
+               points[pindex].y = y;
+               pindex += 1;
             }
          }
          srcx += scalex;
       }
+      JXDrawPoints(display, np->mask, maskGC, points, pindex, CoordModeOrigin);
       srcy += scaley;
    }
+   Release(points);
 
    /* Release the mask GC. */
    JXFreeGC(display, maskGC);
  
    /* Create the color data pixmap. */
-   np->image = JXCreatePixmap(display, rootWindow, nwidth, nheight, rootDepth);
+   np->image = JXCreatePixmap(display, rootWindow, nwidth, nheight,
+                              rootVisual.depth);
 
    /* Render the image to the color data pixmap. */
    JXPutImage(display, np->image, rootGC, image, 0, 0, 0, 0, nwidth, nheight);   
@@ -613,50 +697,56 @@ ScaledIconNode *GetScaledIcon(IconNode *icon, long fg,
 IconNode *CreateIconFromBinary(const unsigned long *input,
                                unsigned int length)
 {
-
-   unsigned long height, width;
-   IconNode *result;
-   unsigned char *data;
-   unsigned int x, index;
+   IconNode *result = NULL;
+   unsigned int offset = 0;
 
    if(!input) {
       return NULL;
    }
 
-   width = input[0];
-   height = input[1];
+   while(offset < length) {
+
+      const unsigned int width = input[offset + 0];
+      const unsigned int height = input[offset + 1];
+      unsigned char *data;
+      ImageNode *image;
+      unsigned int x, index;
+
+      if(JUNLIKELY(width * height + 2 > length - offset)) {
+         Debug("invalid image size: %d x %d + 2 > %d",
+               width, height, length - offset);
+         return result;
+      } else if(JUNLIKELY(width == 0 || height == 0)) {
+         Debug("invalid image size: %d x %d", width, height);
+         return result;
+      }
+
+      if(result == NULL) {
+         result = CreateIcon();
+      }
+
+      image = CreateImage(width, height, 0);
+      image->next = result->images;
+      result->images = image;
+      data = image->data;
+
+      /* Note: the data types here might be of different sizes. */
+      index = 0;
+      for(x = 0; x < width * height; x++) {
+         data[index++] = (input[offset + x + 2] >> 24) & 0xFF;
+         data[index++] = (input[offset + x + 2] >> 16) & 0xFF;
+         data[index++] = (input[offset + x + 2] >>  8) & 0xFF;
+         data[index++] = (input[offset + x + 2] >>  0) & 0xFF;
+      }
+
+      /* Move to the next size. */
+      offset += 2 + width * height;
+
+      /* Don't insert this icon into the hash since it is transient. */
 
-   if(JUNLIKELY(width * height + 2 > length)) {
-      Debug("invalid image size: %d x %d + 2 > %d", width, height, length);
-      return NULL;
-   } else if(JUNLIKELY(width == 0 || height == 0)) {
-      Debug("invalid image size: %d x %d", width, height);
-      return NULL;
    }
 
-   result = CreateIcon();
-
-   result->image = Allocate(sizeof(ImageNode));
-   result->image->width = width;
-   result->image->height = height;
-   result->image->bitmap = 0;
-
-   result->image->data = Allocate(4 * width * height);
-   data = result->image->data;
-
-   /* Note: the data types here might be of different sizes. */
-   index = 0;
-   for(x = 0; x < width * height; x++) {
-      data[index++] = input[x + 2] >> 24;
-      data[index++] = (input[x + 2] >> 16) & 0xFF;
-      data[index++] = (input[x + 2] >> 8) & 0xFF;
-      data[index++] = input[x + 2] & 0xFF;
-   }
-
-   /* Don't insert this icon since it is transient. */
-
    return result;
-
 }
 
 /** Create an empty icon node. */
@@ -665,45 +755,47 @@ IconNode *CreateIcon(void)
    IconNode *icon;
    icon = Allocate(sizeof(IconNode));
    icon->name = NULL;
-   icon->image = NULL;
-   icon->nodes = NULL;
+   icon->images = NULL;
    icon->next = NULL;
    icon->prev = NULL;
+   icon->preserveAspect = 1;
    return icon;
 }
 
 /** Helper method for destroy icons. */
 void DoDestroyIcon(int index, IconNode *icon)
 {
-   ScaledIconNode *np;
    if(icon) {
-      while(icon->nodes) {
-         np = icon->nodes->next;
+      ImageNode *image = icon->images;
+      while(image) {
+         ScaledIconNode *np = image->nodes;
+         while(np) {
+            ScaledIconNode *next_node = np->next;
 
 #ifdef USE_XRENDER
-         if(icon->nodes->imagePicture != None) {
-            JXRenderFreePicture(display, icon->nodes->imagePicture);
-         }
-         if(icon->nodes->alphaPicture != None) {
-            JXRenderFreePicture(display, icon->nodes->alphaPicture);
-         }
+            if(np->imagePicture != None) {
+               JXRenderFreePicture(display, np->imagePicture);
+            }
+            if(np->alphaPicture != None) {
+               JXRenderFreePicture(display, np->alphaPicture);
+            }
 #endif
+            if(np->image != None) {
+               JXFreePixmap(display, np->image);
+            }
+            if(np->mask != None) {
+               JXFreePixmap(display, np->mask);
+            }
 
-         if(icon->nodes->image != None) {
-            JXFreePixmap(display, icon->nodes->image);
+            Release(np);
+            np = next_node;
          }
-         if(icon->nodes->mask != None) {
-            JXFreePixmap(display, icon->nodes->mask);
-         }
-
-         Release(icon->nodes);
-         icon->nodes = np;
+         image = image->next;
       }
-
       if(icon->name) {
          Release(icon->name);
       }
-      DestroyImage(icon->image);
+      DestroyImage(icon->images);
 
       if(icon->prev) {
          icon->prev->next = icon->next;
