@@ -11,6 +11,7 @@
 #define ICON_H
 
 struct ClientNode;
+struct VisualData;
 
 /** Structure to hold a scaled icon. */
 typedef struct ScaledIconNode {
@@ -34,12 +35,14 @@ typedef struct ScaledIconNode {
 typedef struct IconNode {
 
    char *name;                    /**< The name of the icon. */
-   struct ImageNode *image;       /**< The image data. */
-   struct ScaledIconNode *nodes;  /**< Scaled versions of the icon. */
+   struct ImageNode *images;      /**< Images associated with this icon. */
 
    struct IconNode *next;         /**< The next icon in the list. */
    struct IconNode *prev;         /**< The previous icon in the list. */
 
+   char preserveAspect;           /**< Set to preserve the aspect ratio
+                                   *   of the icon when scaling. */
+
 } IconNode;
 
 extern IconNode emptyIcon;
@@ -62,6 +65,7 @@ void AddIconPath(char *path);
 /** Render an icon.
  * This will scale an icon if necessary to fit the requested size. The
  * aspect ratio of the icon is preserved.
+ * @param visual Visual data to use.
  * @param icon The icon to render.
  * @param d The drawable on which to place the icon.
  * @param fg The foreground color.
@@ -70,8 +74,8 @@ void AddIconPath(char *path);
  * @param width The width of the icon to display.
  * @param height The height of the icon to display.
  */
-void PutIcon(IconNode *icon, Drawable d, long fg,
-             int x, int y, int width, int height);
+void PutIcon(const struct VisualData *visual, IconNode *icon, Drawable d,
+             long fg, int x, int y, int width, int height);
 
 /** Load an icon for a client.
  * @param np The client.
@@ -81,9 +85,10 @@ void LoadIcon(struct ClientNode *np);
 /** Load an icon.
  * @param name The name of the icon to load.
  * @param save Set if this icon should be saved in the icon hash.
+ * @param preserveAspect Set to preserve the aspect ratio when scaling.
  * @return A pointer to the icon (NULL if not found).
  */
-IconNode *LoadNamedIcon(const char *name, char save);
+IconNode *LoadNamedIcon(const char *name, char save, char preserveAspect);
 
 /** Destroy an icon.
  * @param icon The icon to destroy.
@@ -94,15 +99,15 @@ void DestroyIcon(IconNode *icon);
 
 #define ICON_DUMMY_FUNCTION ((void)0)
 
-#define InitializeIcons()               ICON_DUMMY_FUNCTION
-#define StartupIcons()                  ICON_DUMMY_FUNCTION
-#define ShutdownIcons()                 ICON_DUMMY_FUNCTION
-#define DestroyIcons()                  ICON_DUMMY_FUNCTION
-#define AddIconPath( a )                ICON_DUMMY_FUNCTION
-#define PutIcon( a, b, c, d, e, f, g )  ICON_DUMMY_FUNCTION
-#define LoadIcon( a )                   ICON_DUMMY_FUNCTION
-#define LoadNamedIcon( a, b )           NULL
-#define DestroyIcon( a )                ICON_DUMMY_FUNCTION
+#define InitializeIcons()                  ICON_DUMMY_FUNCTION
+#define StartupIcons()                     ICON_DUMMY_FUNCTION
+#define ShutdownIcons()                    ICON_DUMMY_FUNCTION
+#define DestroyIcons()                     ICON_DUMMY_FUNCTION
+#define AddIconPath( a )                   ICON_DUMMY_FUNCTION
+#define PutIcon( a, b, c, d, e, f, g, h )  ICON_DUMMY_FUNCTION
+#define LoadIcon( a )                      ICON_DUMMY_FUNCTION
+#define LoadNamedIcon( a, b, c )           NULL
+#define DestroyIcon( a )                   ICON_DUMMY_FUNCTION
 
 #endif /* USE_ICONS */
 
