@@ -51,10 +51,8 @@ static ImageNode *LoadXPMImage(const char *fileName);
 static ImageNode *LoadXBMImage(const char *fileName);
 #endif
 #ifdef USE_ICONS
-#ifdef USE_XPM
 static ImageNode *CreateImageFromXImages(XImage *image, XImage *shape);
 #endif
-#endif
 
 #ifdef USE_XPM
 static int AllocateColor(Display *d, Colormap cmap, char *name,
@@ -153,6 +151,29 @@ ImageNode *LoadImageFromData(char **data)
 
 }
 
+/** Load an image from a pixmap. */
+#ifdef USE_ICONS
+ImageNode *LoadImageFromDrawable(Drawable pmap, Pixmap mask)
+{
+   XImage *mask_image = NULL;
+   XImage *icon_image;
+   Window rwindow;
+   int x, y;
+   unsigned int width, height;
+   unsigned int border_width;
+   unsigned int depth;
+
+   JXGetGeometry(display, pmap, &rwindow, &x, &y, &width, &height,
+                 &border_width, &depth);
+   icon_image = JXGetImage(display, pmap, 0, 0, width, height,
+                           AllPlanes, ZPixmap);
+   if(mask != None) {
+      mask_image = JXGetImage(display, mask, 0, 0, width, height, 1, ZPixmap);
+   }
+   return CreateImageFromXImages(icon_image, mask_image);
+}
+#endif
+
 /** Load a PNG image from the given file name.
  * Since libpng uses longjmp, this function is not reentrant to simplify
  * the issues surrounding longjmp and local variables.
@@ -236,14 +257,9 @@ ImageNode *LoadPNGImage(const char *fileName)
 
    png_read_info(pngData, pngInfo);
 
-   result = Allocate(sizeof(ImageNode));
-   result->data = NULL;
-   result->bitmap = 0;
-
    png_get_IHDR(pngData, pngInfo, &width, &height,
                 &bitDepth, &colorType, NULL, NULL, NULL);
-   result->width = (int)width;
-   result->height = (int)height;
+   result = CreateImage(width, height, 0);
 
    png_set_expand(pngData);
 
@@ -264,8 +280,6 @@ ImageNode *LoadPNGImage(const char *fileName)
    png_read_update_info(pngData, pngInfo);
 
    rowBytes = png_get_rowbytes(pngData, pngInfo);
-   result->data = Allocate(rowBytes * result->height);
-
    rows = AllocateStack(result->height * sizeof(result->data));
    y = 0;
    for(x = 0; x < result->height; x++) {
@@ -349,11 +363,8 @@ ImageNode *LoadJPEGImage(const char *fileName)
    buffer = (*cinfo.mem->alloc_sarray)((j_common_ptr)&cinfo,
                                        JPOOL_IMAGE, rowStride, 1);
 
-   result = Allocate(sizeof(ImageNode));
-   result->width = cinfo.image_width;
-   result->height = cinfo.image_height;
+   result = CreateImage(cinfo.image_width, cinfo.image_height, 0);
    result->data = Allocate(4 * result->width * result->height);
-   result->bitmap = 0;
 
    /* Read lines. */
    outIndex = 0;
@@ -424,12 +435,8 @@ ImageNode *LoadSVGImage(const char *fileName)
 
    rsvg_handle_get_dimensions(rh, &dim);
 
-   result = (ImageNode*)Allocate(sizeof(ImageNode));
-   result->bitmap = 0;
-   result->width = dim.width;
-   result->height = dim.height;
-   result->data = Allocate(4 * dim.width * dim.height);
-   memset(result->data, 0, 4 * dim.width * dim.height);
+   result = CreateImage(dim.width, dim.height, 0);
+   memset(result->data, 0, dim.width * dim.height * 4);
 
    /* Create the target surface. */
    stride = cairo_format_stride_for_width(CAIRO_FORMAT_ARGB32, dim.width);
@@ -444,11 +451,11 @@ ImageNode *LoadSVGImage(const char *fileName)
    g_object_unref(rh);
 
    for(i = 0; i < 4 * dim.width * dim.height; i += 4) {
-      const unsigned long temp = *(unsigned long*)&result->data[i];
-      const unsigned long alpha  = (temp >> 24) & 0xFF;
-      const unsigned long red    = (temp >> 16) & 0xFF;
-      const unsigned long green  = (temp >>  8) & 0xFF;
-      const unsigned long blue   = (temp >>  0) & 0xFF;
+      const unsigned int temp = *(unsigned int*)&result->data[i];
+      const unsigned int alpha  = (temp >> 24) & 0xFF;
+      const unsigned int red    = (temp >> 16) & 0xFF;
+      const unsigned int green  = (temp >>  8) & 0xFF;
+      const unsigned int blue   = (temp >>  0) & 0xFF;
       result->data[i + 0] = alpha;
       if(alpha > 0) {
          result->data[i + 1] = (red * 255) / alpha;
@@ -507,12 +514,7 @@ ImageNode *LoadXBMImage(const char *fileName)
 
    rc = XReadBitmapFileData(fileName, &width, &height, &data, &hotx, &hoty);
    if(rc == BitmapSuccess) {
-      result = (ImageNode*)Allocate(sizeof(ImageNode));
-      result->bitmap = 1;
-      result->width = width;
-      result->height = height;
-      result->data = Allocate((width * height) / 8);
-      memcpy(result->data, data, (width * height) / 8);
+      result = CreateImage(width, height, 1);
       XFree(data);
    }
 
@@ -522,7 +524,6 @@ ImageNode *LoadXBMImage(const char *fileName)
 
 /** Create an image from XImages giving color and shape information. */
 #ifdef USE_ICONS
-#ifdef USE_XPM
 ImageNode *CreateImageFromXImages(XImage *image, XImage *shape)
 {
 
@@ -532,12 +533,7 @@ ImageNode *CreateImageFromXImages(XImage *image, XImage *shape)
    int index;
    int x, y;
 
-   result = Allocate(sizeof(ImageNode));
-   result->data = Allocate(4 * image->width * image->height);
-   result->width = image->width;
-   result->height = image->height;
-   result->bitmap = 0;
-
+   result = CreateImage(image->width, image->height, 0);
    index = 0;
    for(y = 0; y < image->height; y++) {
       for(x = 0; x < image->width; x++) {
@@ -565,16 +561,35 @@ ImageNode *CreateImageFromXImages(XImage *image, XImage *shape)
    return result;
 
 }
-#endif /* USE_XPM */
 #endif /* USE_ICONS */
 
+ImageNode *CreateImage(unsigned int width, unsigned int height, char bitmap)
+{
+   unsigned int image_size;
+   if(bitmap) {
+      image_size = (width * height + 7) / 8;
+   } else {
+      image_size = 4 * width * height;
+   }
+   ImageNode *image = Allocate(sizeof(ImageNode));
+   image->data = Allocate(image_size);
+   image->next = NULL;
+   image->nodes = NULL;
+   image->bitmap = 0;
+   image->width = width;
+   image->height = height;
+   return image;
+}
+
 /** Destroy an image node. */
 void DestroyImage(ImageNode *image) {
-   if(image) {
+   while(image) {
+      ImageNode *next = image->next;
       if(image->data) {
          Release(image->data);
       }
       Release(image);
+      image = next;
    }
 }
 
@@ -606,4 +621,3 @@ int FreeColors(Display *d, Colormap cmap, Pixel *pixels, int n,
    return 1;
 }
 #endif /* USE_XPM */
-
