@@ -15,83 +15,81 @@
 /** Amount to increase allocations by when reading text. */
 static const int BLOCK_SIZE = 16;
 
-/** Literal names for tokens.
- * This order is important. It must match the order of the enumeration
- * in lex.h.
+/** Mapping between token names and tokens.
+ * These must be sorted.
  */
-static const char *TOKEN_MAP[] = {
-   "[invalid]",
-   "Active",
-   "ActiveBackground",
-   "ActiveForeground",
-   "Background",
-   "Border",
-   "ButtonClose",
-   "ButtonMax",
-   "ButtonMaxActive",
-   "ButtonMin",
-   "Class",
-   "Clock",
-   "ClockStyle",
-   "Close",
-   "Desktops",
-   "Desktop",
-   "Dock",
-   "DoubleClickSpeed",
-   "DoubleClickDelta",
-   "Exit",
-   "FocusModel",
-   "Font",
-   "Foreground",
-   "Group",
-   "Height",
-   "IconPath",
-   "Inactive",
-   "Include",
-   "JWM",
-   "Key",
-   "Kill",
-   "Layer",
-   "Maximize",
-   "Menu",
-   "MenuStyle",
-   "Minimize",
-   "Move",
-   "MoveMode",
-   "Name",
-   "Opacity",
-   "Option",
-   "Outline",
-   "Pager",
-   "PagerStyle",
-   "Popup",
-   "PopupStyle",
-   "Program",
-   "Resize",
-   "ResizeMode",
-   "Restart",
-   "RestartCommand",
-   "RootMenu",
-   "SendTo",
-   "Separator",
-   "Shade",
-   "ShutdownCommand",
-   "SnapMode",
-   "Spacer",
-   "StartupCommand",
-   "Stick",
-   "Swallow",
-   "TaskListStyle",
-   "TaskList",
-   "Text",
-   "Title",
-   "Tray",
-   "TrayButton",
-   "TrayButtonStyle",
-   "TrayStyle",
-   "Width",
-   "WindowStyle"
+static const StringMappingType TOKEN_MAP[] = {
+   { "Active",             TOK_ACTIVE           },
+   { "Background",         TOK_BACKGROUND       },
+   { "Button",             TOK_BUTTON           },
+   { "ButtonClose",        TOK_BUTTONCLOSE      },
+   { "ButtonMax",          TOK_BUTTONMAX        },
+   { "ButtonMaxActive",    TOK_BUTTONMAXACTIVE  },
+   { "ButtonMin",          TOK_BUTTONMIN        },
+   { "Class",              TOK_CLASS            },
+   { "Clock",              TOK_CLOCK            },
+   { "ClockStyle",         TOK_CLOCKSTYLE       },
+   { "Close",              TOK_CLOSE            },
+   { "Corner",             TOK_CORNER           },
+   { "Desktop",            TOK_DESKTOP          },
+   { "Desktops",           TOK_DESKTOPS         },
+   { "Dock",               TOK_DOCK             },
+   { "DoubleClickDelta",   TOK_DOUBLECLICKDELTA },
+   { "DoubleClickSpeed",   TOK_DOUBLECLICKSPEED },
+   { "Dynamic",            TOK_DYNAMIC          },
+   { "Exit",               TOK_EXIT             },
+   { "FocusModel",         TOK_FOCUSMODEL       },
+   { "Font",               TOK_FONT             },
+   { "Foreground",         TOK_FOREGROUND       },
+   { "Group",              TOK_GROUP            },
+   { "Height",             TOK_HEIGHT           },
+   { "IconPath",           TOK_ICONPATH         },
+   { "Include",            TOK_INCLUDE          },
+   { "JWM",                TOK_JWM              },
+   { "Key",                TOK_KEY              },
+   { "Kill",               TOK_KILL             },
+   { "Layer",              TOK_LAYER            },
+   { "Maximize",           TOK_MAXIMIZE         },
+   { "Menu",               TOK_MENU             },
+   { "MenuStyle",          TOK_MENUSTYLE        },
+   { "Minimize",           TOK_MINIMIZE         },
+   { "Move",               TOK_MOVE             },
+   { "MoveMode",           TOK_MOVEMODE         },
+   { "Name",               TOK_NAME             },
+   { "Opacity",            TOK_OPACITY          },
+   { "Option",             TOK_OPTION           },
+   { "Outline",            TOK_OUTLINE          },
+   { "Pager",              TOK_PAGER            },
+   { "PagerStyle",         TOK_PAGERSTYLE       },
+   { "Popup",              TOK_POPUP            },
+   { "PopupStyle",         TOK_POPUPSTYLE       },
+   { "Program",            TOK_PROGRAM          },
+   { "Resize",             TOK_RESIZE           },
+   { "ResizeMode",         TOK_RESIZEMODE       },
+   { "Restart",            TOK_RESTART          },
+   { "RestartCommand",     TOK_RESTARTCOMMAND   },
+   { "RootMenu",           TOK_ROOTMENU         },
+   { "SendTo",             TOK_SENDTO           },
+   { "Separator",          TOK_SEPARATOR        },
+   { "Shade",              TOK_SHADE            },
+   { "ShutdownCommand",    TOK_SHUTDOWNCOMMAND  },
+   { "SnapMode",           TOK_SNAPMODE         },
+   { "Spacer",             TOK_SPACER           },
+   { "StartupCommand",     TOK_STARTUPCOMMAND   },
+   { "Stick",              TOK_STICK            },
+   { "Swallow",            TOK_SWALLOW          },
+   { "TaskList",           TOK_TASKLIST         },
+   { "TaskListStyle",      TOK_TASKLISTSTYLE    },
+   { "Text",               TOK_TEXT             },
+   { "Tray",               TOK_TRAY             },
+   { "TrayButton",         TOK_TRAYBUTTON       },
+   { "TrayButtonStyle",    TOK_TRAYBUTTONSTYLE  },
+   { "TrayStyle",          TOK_TRAYSTYLE        },
+   { "Width",              TOK_WIDTH            },
+   { "WindowStyle",        TOK_WINDOWSTYLE      }
 };
+static const unsigned int TOKEN_MAP_COUNT
+   = sizeof(TOKEN_MAP) / sizeof(TOKEN_MAP[0]);
 
 static TokenNode *head;
 
@@ -121,7 +119,7 @@ static int ParseEntity(const char *entity, char *ch,
                        const char *file, unsigned int line);
 static TokenType LookupType(const char *name, TokenNode *np);
 
-/** Tokenize a data. */
+/** Tokenize data. */
 TokenNode *Tokenize(const char *line, const char *fileName)
 {
    AttributeNode *ap;
@@ -487,17 +485,12 @@ char *ReadAttributeValue(const char *line,
 /** Get the token for a tag name. */
 TokenType LookupType(const char *name, TokenNode *np)
 {
-   unsigned int x;
-
-   Assert(name);
-
-   for(x = 0; x < sizeof(TOKEN_MAP) / sizeof(char*); x++) {
-      if(!strcmp(name, TOKEN_MAP[x])) {
-         if(np) {
-            np->type = x;
-         }
-         return x;
+   const int x = FindValue(TOKEN_MAP, TOKEN_MAP_COUNT, name);
+   if(x >= 0) {
+      if(np) {
+         np->type = x;
       }
+      return x;
    }
 
    if(JUNLIKELY(np)) {
@@ -514,16 +507,15 @@ const char *GetTokenName(const TokenNode *tp)
 {
    if(tp->invalidName) {
       return tp->invalidName;
-   } else if(tp->type >= sizeof(TOKEN_MAP) / sizeof(const char*)) {
-      return "[invalid]";
    } else {
-      return TOKEN_MAP[tp->type];
+      return GetTokenTypeName(tp->type);
    }
 }
 
 /** Get the string representation of a token. */
 const char *GetTokenTypeName(TokenType type) {
-   return TOKEN_MAP[type];
+   const char *key = FindKey(TOKEN_MAP, TOKEN_MAP_COUNT, type);
+   return key ? key : "[invalid]";
 }
 
 /** Create an empty XML tag node. */
