@@ -10,19 +10,14 @@
 #ifndef LEX_H
 #define LEX_H
 
-/** Tokens.
- * Note that any change made to this typedef must be reflected in
- * TOKEN_MAP in lex.c.
- */
+/** Tokens.  */
 typedef enum {
 
    TOK_INVALID,
 
    TOK_ACTIVE,
-   TOK_ACTIVEBACKGROUND,
-   TOK_ACTIVEFOREGROUND,
    TOK_BACKGROUND,
-   TOK_BORDER,
+   TOK_BUTTON,
    TOK_BUTTONCLOSE,
    TOK_BUTTONMAX,
    TOK_BUTTONMAXACTIVE,
@@ -31,11 +26,13 @@ typedef enum {
    TOK_CLOCK,
    TOK_CLOCKSTYLE,
    TOK_CLOSE,
-   TOK_DESKTOPS,
+   TOK_CORNER,
    TOK_DESKTOP,
+   TOK_DESKTOPS,
    TOK_DOCK,
-   TOK_DOUBLECLICKSPEED,
    TOK_DOUBLECLICKDELTA,
+   TOK_DOUBLECLICKSPEED,
+   TOK_DYNAMIC,
    TOK_EXIT,
    TOK_FOCUSMODEL,
    TOK_FONT,
@@ -43,7 +40,6 @@ typedef enum {
    TOK_GROUP,
    TOK_HEIGHT,
    TOK_ICONPATH,
-   TOK_INACTIVE,
    TOK_INCLUDE,
    TOK_JWM,
    TOK_KEY,
@@ -78,10 +74,9 @@ typedef enum {
    TOK_STARTUPCOMMAND,
    TOK_STICK,
    TOK_SWALLOW,
-   TOK_TASKLISTSTYLE,
    TOK_TASKLIST,
+   TOK_TASKLISTSTYLE,
    TOK_TEXT,
-   TOK_TITLE,
    TOK_TRAY,
    TOK_TRAYBUTTON,
    TOK_TRAYBUTTONSTYLE,
