@@ -9,7 +9,6 @@
 
 #include "jwm.h"
 #include "main.h"
-#include "lex.h"
 #include "parse.h"
 #include "help.h"
 #include "error.h"
@@ -22,7 +21,6 @@
 #include "cursor.h"
 #include "confirm.h"
 #include "font.h"
-#include "hint.h"
 #include "group.h"
 #include "key.h"
 #include "icon.h"
@@ -47,12 +45,13 @@
 Display *display = NULL;
 Window rootWindow;
 int rootWidth, rootHeight;
-int rootDepth;
 int rootScreen;
 Colormap rootColormap;
-Visual *rootVisual;
+VisualData rootVisual;
 GC rootGC;
 int colormapCount;
+Window supportingWindow;
+Atom managerSelection;
 
 char shouldExit = 0;
 char shouldRestart = 0;
@@ -84,6 +83,7 @@ static void Destroy(void);
 
 static void OpenConnection(void);
 static void CloseConnection(void);
+static Bool SelectionReleased(Display *d, XEvent *e, XPointer arg);
 static void StartupConnection(void);
 static void ShutdownConnection(void);
 static void EventLoop(void);
@@ -107,7 +107,8 @@ int main(int argc, char *argv[])
       ACTION_RUN,
       ACTION_RESTART,
       ACTION_EXIT,
-      ACTION_RELOAD
+      ACTION_RELOAD,
+      ACTION_PARSE
    } action;
 
    StartDebug();
@@ -132,17 +133,18 @@ int main(int argc, char *argv[])
          DisplayHelp();
          DoExit(0);
       } else if(!strcmp(argv[x], "-p")) {
-         Initialize();
-         ParseConfig(configPath);
-         DoExit(0);
+         action = ACTION_PARSE;
       } else if(!strcmp(argv[x], "-restart")) {
          action = ACTION_RESTART;
       } else if(!strcmp(argv[x], "-exit")) {
          action = ACTION_EXIT;
-		} else if(!strcmp(argv[x], "-reload")) {
+      } else if(!strcmp(argv[x], "-reload")) {
          action = ACTION_RELOAD;
       } else if(!strcmp(argv[x], "-display") && x + 1 < argc) {
          displayString = argv[++x];
+      } else if(!strcmp(argv[x], "-f") && x + 1 < argc) {
+         Release(configPath);
+         configPath = CopyString(argv[++x]);
       } else {
          printf("unrecognized option: %s\n", argv[x]);
          DisplayHelp();
@@ -151,6 +153,10 @@ int main(int argc, char *argv[])
    }
 
    switch(action) {
+   case ACTION_PARSE:
+      Initialize();
+      ParseConfig(configPath);
+      DoExit(0);
    case ACTION_RESTART:
       SendRestart();
       DoExit(0);
@@ -288,9 +294,9 @@ void OpenConnection(void)
    rootWindow = RootWindow(display, rootScreen);
    rootWidth = DisplayWidth(display, rootScreen);
    rootHeight = DisplayHeight(display, rootScreen);
-   rootDepth = DefaultDepth(display, rootScreen);
+   rootVisual.depth = DefaultDepth(display, rootScreen);
+   rootVisual.visual = DefaultVisual(display, rootScreen);
    rootColormap = DefaultColormap(display, rootScreen);
-   rootVisual = DefaultVisual(display, rootScreen);
    rootGC = DefaultGC(display, rootScreen);
    colormapCount = MaxCmapsOfScreen(ScreenOfDisplay(display, rootScreen));
 
@@ -299,6 +305,17 @@ void OpenConnection(void)
 
 }
 
+/** Predicate for XIfEvent to determine if we got the WM_Sn selection. */
+Bool SelectionReleased(Display *d, XEvent *e, XPointer arg)
+{
+   if(e->type == DestroyNotify) {
+      if(e->xdestroywindow.window == *(Window*)arg) {
+         return True;
+      }
+   }
+   return False;
+}
+
 /** Prepare the connection. */
 void StartupConnection(void)
 {
@@ -312,6 +329,10 @@ void StartupConnection(void)
    int renderError;
 #endif
    struct sigaction sa;
+   char name[32];
+   Window win;
+   XEvent event;
+   int revert;
 
    initializing = 1;
    OpenConnection();
@@ -320,6 +341,45 @@ void StartupConnection(void)
    XSynchronize(display, True);
 #endif
 
+   /* Create the supporting window used to verify JWM is running. */
+   supportingWindow = JXCreateSimpleWindow(display, rootWindow,
+                                           0, 0, 1, 1, 0, 0, 0);
+
+   /* Get the atom used for the window manager selection. */
+   snprintf(name, 32, "WM_S%d", rootScreen);
+   managerSelection = JXInternAtom(display, name, False);
+
+   /* Get the current window manager and take the selection. */
+   GrabServer();
+   win = JXGetSelectionOwner(display, managerSelection);
+   if(win != None) {
+      JXSelectInput(display, win, StructureNotifyMask);
+   }
+   JXSetSelectionOwner(display, managerSelection,
+                       supportingWindow, CurrentTime);
+   UngrabServer();
+
+   /* Wait for the current selection owner to give up the selection. */
+   if(win != None) {
+      /* Note that we need to wait for the current selection owner
+       * to exit before we can expect to select SubstructureRedirectMask. */
+      XIfEvent(display, &event, SelectionReleased, (XPointer)&win);
+      JXSync(display, False);
+   }
+
+   event.xclient.display = display;
+   event.xclient.type = ClientMessage;
+   event.xclient.window = rootWindow;
+   event.xclient.message_type = JXInternAtom(display, "MANAGER", False);
+   event.xclient.format = 32;
+   event.xclient.data.l[0] = CurrentTime;
+   event.xclient.data.l[1] = managerSelection;
+   event.xclient.data.l[2] = supportingWindow;
+   event.xclient.data.l[3] = 2;
+   event.xclient.data.l[4] = 0;
+   JXSendEvent(display, rootWindow, False, StructureNotifyMask, &event);
+   JXSync(display, False);
+
    JXSetErrorHandler(ErrorHandler);
 
    clientContext = XUniqueContext();
@@ -369,6 +429,13 @@ void StartupConnection(void)
    }
 #endif
 
+   /* Make sure we have input focus. */
+   win = None;
+   JXGetInputFocus(display, &win, &revert);
+   if(win == None) {
+      JXSetInputFocus(display, rootWindow, RevertToParent, CurrentTime);
+   }
+
    initializing = 0;
 
 }
@@ -473,7 +540,7 @@ void Startup(void)
    ReadCurrentDesktop();
    JXFlush(display);
 
-   RestackClients();
+   RequireRestack();
 
    /* Allow clients to do their thing. */
    JXSync(display, True);
@@ -573,19 +640,19 @@ void Destroy(void)
 /** Send _JWM_RESTART to the root window. */
 void SendRestart(void)
 {
-	SendJWMMessage(jwmRestart);
+   SendJWMMessage(jwmRestart);
 }
 
 /** Send _JWM_EXIT to the root window. */
 void SendExit(void)
 {
-	SendJWMMessage(jwmExit);
+   SendJWMMessage(jwmExit);
 }
 
 /** Send _JWM_RELOAD to the root window. */
 void SendReload(void)
 {
-	SendJWMMessage(jwmReload);
+   SendJWMMessage(jwmReload);
 }
 
 /** Send a JWM message to the root window. */
