@@ -11,34 +11,40 @@
 #include "menu.h"
 #include "font.h"
 #include "client.h"
-#include "color.h"
 #include "icon.h"
-#include "image.h"
-#include "main.h"
 #include "cursor.h"
 #include "key.h"
 #include "button.h"
 #include "event.h"
-#include "error.h"
 #include "root.h"
 #include "settings.h"
+#include "desktop.h"
+#include "parse.h"
+#include "winmenu.h"
+#include "screen.h"
 
-#define MENU_BORDER_SIZE   1
 #define BASE_ICON_OFFSET   3
+#define MENU_BORDER_SIZE   1
 
 typedef unsigned char MenuSelectionType;
 #define MENU_NOSELECTION   0
 #define MENU_LEAVE         1
 #define MENU_SUBSELECT     2
 
-static char ShowSubmenu(Menu *menu, Menu *parent, int x, int y);
+static char ShowSubmenu(Menu *menu, Menu *parent,
+                        RunMenuCommandType runner,
+                        int x, int y);
 
+static void PatchMenu(Menu *menu);
+static void UnpatchMenu(Menu *menu);
 static void CreateMenu(Menu *menu, int x, int y);
 static void HideMenu(Menu *menu);
 static void DrawMenu(Menu *menu);
 
-static char MenuLoop(Menu *menu);
-static MenuSelectionType UpdateMotion(Menu *menu, XEvent *event);
+static char MenuLoop(Menu *menu, RunMenuCommandType runner);
+static MenuSelectionType UpdateMotion(Menu *menu,
+                                      RunMenuCommandType runner,
+                                      XEvent *event);
 
 static void UpdateMenu(Menu *menu);
 static void DrawMenuItem(Menu *menu, MenuItem *item, int index);
@@ -49,10 +55,17 @@ static int GetMenuIndex(Menu *menu, int index);
 static void SetPosition(Menu *tp, int index);
 static char IsMenuValid(const Menu *menu);
 
-static MenuAction *menuAction = NULL;
-
 int menuShown = 0;
 
+/** Create an empty menu item. */
+MenuItem *CreateMenuItem(MenuItemType type)
+{
+   MenuItem *item = Allocate(sizeof(MenuItem));
+   memset(item, 0, sizeof(MenuItem));
+   item->type = type;
+   return item;
+}
+
 /** Initialize a menu. */
 void InitializeMenu(Menu *menu)
 {
@@ -75,12 +88,12 @@ void InitializeMenu(Menu *menu)
    menu->itemHeight = GetStringHeight(FONT_MENU);
    for(np = menu->items; np; np = np->next) {
       if(np->iconName) {
-         np->icon = LoadNamedIcon(np->iconName, 1);
+         np->icon = LoadNamedIcon(np->iconName, 1, 1);
          if(np->icon) {
             hasIcon = 1;
          }
-      } else {
-         np->icon = NULL;
+      } else if(np->icon) {
+         hasIcon = 1;
       }
       menu->itemCount += 1;
    }
@@ -105,7 +118,7 @@ void InitializeMenu(Menu *menu)
       }
    }
 
-   menu->height = MENU_BORDER_SIZE;
+   menu->height = 0;
    if(menu->label) {
       menu->height += menu->itemHeight;
    }
@@ -141,9 +154,7 @@ void InitializeMenu(Menu *menu)
       }
    }
    menu->width += hasSubmenu + menu->textOffset;
-   menu->width += 2 * MENU_BORDER_SIZE;
    menu->width += 7;
-   menu->height += MENU_BORDER_SIZE;
 
 }
 
@@ -168,17 +179,13 @@ void ShowMenu(Menu *menu, RunMenuCommandType runner, int x, int y)
       return;
    }
 
-   ShowSubmenu(menu, NULL, x - MENU_BORDER_SIZE, y - MENU_BORDER_SIZE);
+   ShowSubmenu(menu, NULL, runner, x, y);
+   UnpatchMenu(menu);
 
    JXUngrabKeyboard(display, CurrentTime);
    JXUngrabPointer(display, CurrentTime);
    RefocusClient();
 
-   if(menuAction) {
-      (runner)(menuAction);
-      menuAction = NULL;
-   }
-
    if(shouldReload) {
       ReloadMenu();
    }
@@ -195,9 +202,10 @@ void DestroyMenu(Menu *menu)
          if(menu->items->name) {
             Release(menu->items->name);
          }
-         switch(menu->items->action.type) {
+         switch(menu->items->action.type & MA_ACTION_MASK) {
          case MA_EXECUTE:
          case MA_EXIT:
+         case MA_DYNAMIC:
             if(menu->items->action.data.str) {
                Release(menu->items->action.data.str);
             }
@@ -225,16 +233,19 @@ void DestroyMenu(Menu *menu)
 }
 
 /** Show a submenu. */
-char ShowSubmenu(Menu *menu, Menu *parent, int x, int y)
+char ShowSubmenu(Menu *menu, Menu *parent,
+                 RunMenuCommandType runner,
+                 int x, int y)
 {
 
    char status;
 
+   PatchMenu(menu);
    menu->parent = parent;
    CreateMenu(menu, x, y);
 
    menuShown += 1;
-   status = MenuLoop(menu);
+   status = MenuLoop(menu, runner);
    menuShown -= 1;
 
    HideMenu(menu);
@@ -243,10 +254,66 @@ char ShowSubmenu(Menu *menu, Menu *parent, int x, int y)
 
 }
 
+/** Prepare a menu to be shown. */
+void PatchMenu(Menu *menu)
+{
+   MenuItem *item;
+   for(item = menu->items; item; item = item->next) {
+      Menu *submenu = NULL;
+      switch(item->action.type & MA_ACTION_MASK) {
+      case MA_DESKTOP_MENU:
+         submenu = CreateDesktopMenu(1 << currentDesktop,
+                                     item->action.context);
+         break;
+      case MA_SENDTO_MENU:
+         submenu = CreateSendtoMenu(
+            item->action.type & ~MA_ACTION_MASK,
+            item->action.context);
+         break;
+      case MA_WINDOW_MENU:
+         submenu = CreateWindowMenu(item->action.context);
+         break;
+      case MA_DYNAMIC:
+         if(!item->submenu) {
+            submenu = ParseDynamicMenu(item->action.data.str);
+         }
+         break;
+      default:
+         break;
+      }
+      if(submenu) {
+         InitializeMenu(submenu);
+         item->submenu = submenu;
+      }
+   }
+}
+
+/** Remove temporary items from a menu. */
+void UnpatchMenu(Menu *menu)
+{
+   MenuItem *item;
+   for(item = menu->items; item; item = item->next) {
+      if(item->submenu) {
+         UnpatchMenu(item->submenu);
+         switch(item->action.type & MA_ACTION_MASK) {
+         case MA_DESKTOP_MENU:
+         case MA_SENDTO_MENU:
+         case MA_WINDOW_MENU:
+         case MA_DYNAMIC:
+            DestroyMenu(item->submenu);
+            item->submenu = NULL;
+            break;
+         default:
+            break;
+         }
+      }
+   }
+}
+
 /** Menu process loop.
  * Returns 0 if no selection was made or 1 if a selection was made.
  */
-char MenuLoop(Menu *menu)
+char MenuLoop(Menu *menu, RunMenuCommandType runner)
 {
 
    XEvent event;
@@ -285,7 +352,7 @@ char MenuLoop(Menu *menu)
       case KeyPress:
       case MotionNotify:
          hadMotion = 1;
-         switch(UpdateMotion(menu, &event)) {
+         switch(UpdateMotion(menu, runner, &event)) {
          case MENU_NOSELECTION: /* no selection */
             break;
          case MENU_LEAVE: /* mouse left the menu */
@@ -312,10 +379,15 @@ char MenuLoop(Menu *menu)
                break;
             }
          }
-            
-         ip = GetMenuItem(menu, menu->currentIndex);
+
+         if(menu->parent && menu->currentIndex < 0) {
+            ip = GetMenuItem(menu->parent, menu->parent->currentIndex);
+         } else {
+            ip = GetMenuItem(menu, menu->currentIndex);
+         }
          if(ip != NULL) {
-            menuAction = &ip->action;
+            ip->action.button = event.xbutton.button;
+            (runner)(&ip->action);
          }
          return 1;
       default:
@@ -336,16 +408,22 @@ void CreateMenu(Menu *menu, int x, int y)
    menu->lastIndex = -1;
    menu->currentIndex = -1;
 
-   if(x + menu->width > rootWidth) {
+   if(menu->parent) {
+      menu->screen = menu->parent->screen;
+   } else {
+      menu->screen = GetCurrentScreen(x + menu->width / 2,
+                                      y + menu->height / 2);
+   }
+   if(x + menu->width > menu->screen->x + menu->screen->width) {
       if(menu->parent) {
-         x = menu->parent->x - menu->width + MENU_BORDER_SIZE;
+         x = menu->parent->x - menu->width;
       } else {
-         x = rootWidth - menu->width;
+         x = menu->screen->x + menu->screen->width - menu->width;
       }
    }
    temp = y;
-   if(y + menu->height > rootHeight) {
-      y = rootHeight - menu->height;
+   if(y + menu->height > menu->screen->y + menu->screen->height) {
+      y = menu->screen->y + menu->screen->height - menu->height;
    }
    if(y < 0) {
       y = 0;
@@ -366,12 +444,16 @@ void CreateMenu(Menu *menu, int x, int y)
    attrMask |= CWSaveUnder;
    attr.save_under = True;
 
+   attrMask |= CWBorderPixel;
+   attr.border_pixel = colors[COLOR_MENU_OUTLINE];
+
    menu->window = JXCreateWindow(display, rootWindow, x, y,
-                                 menu->width, menu->height, 0,
+                                 menu->width, menu->height,
+                                 MENU_BORDER_SIZE,
                                  CopyFromParent, InputOutput,
                                  CopyFromParent, attrMask, &attr);
    menu->pixmap = JXCreatePixmap(display, menu->window,
-                                 menu->width, menu->height, rootDepth);
+                                 menu->width, menu->height, rootVisual.depth);
 
    if(settings.menuOpacity < UINT_MAX) {
       SetCardinalAtom(menu->window, ATOM_NET_WM_WINDOW_OPACITY,
@@ -399,9 +481,6 @@ void DrawMenu(Menu *menu)
    JXSetForeground(display, rootGC, colors[COLOR_MENU_BG]);
    JXFillRectangle(display, menu->pixmap, rootGC, 0, 0,
                    menu->width, menu->height);
-   JXSetForeground(display, rootGC, colors[COLOR_MENU_DOWN]);
-   JXDrawRectangle(display, menu->pixmap, rootGC, 0, 0,
-                   menu->width - 1, menu->height - 1);
 
    if(menu->label) {
       DrawMenuItem(menu, NULL, -1);
@@ -418,9 +497,10 @@ void DrawMenu(Menu *menu)
 }
 
 /** Determine the action to take given an event. */
-MenuSelectionType UpdateMotion(Menu *menu, XEvent *event)
+MenuSelectionType UpdateMotion(Menu *menu,
+                               RunMenuCommandType runner,
+                               XEvent *event)
 {
-
    MenuItem *ip;
    Menu *tp;
    Window subwindow;
@@ -490,9 +570,10 @@ MenuSelectionType UpdateMotion(Menu *menu, XEvent *event)
       case KEY_ESC:
          return MENU_SUBSELECT;
       case KEY_ENTER:
-         ip = GetMenuItem(menu, tp->currentIndex);
+         ip = GetMenuItem(tp, tp->currentIndex);
          if(ip != NULL) {
-            menuAction = &ip->action;
+            ip->action.button = 0;
+            (runner)(&ip->action);
          }
          return MENU_SUBSELECT;
       default:
@@ -539,7 +620,7 @@ MenuSelectionType UpdateMotion(Menu *menu, XEvent *event)
    }
 
    /* Move the menu if needed. */
-   if(menu->height > rootHeight && menu->currentIndex >= 0) {
+   if(menu->height > menu->screen->height && menu->currentIndex >= 0) {
 
       /* If near the top, shift down. */
       if(y + menu->y <= 0) {
@@ -550,7 +631,8 @@ MenuSelectionType UpdateMotion(Menu *menu, XEvent *event)
       }
 
       /* If near the bottom, shift up. */
-      if(y + menu->y + menu->itemHeight / 2 >= rootHeight) {
+      if(y + menu->y + menu->itemHeight / 2
+            >= menu->screen->y + menu->screen->height) {
          if(menu->currentIndex + 1 < menu->itemCount) {
             menu->currentIndex += 1;
             SetPosition(menu, menu->currentIndex);
@@ -567,8 +649,8 @@ MenuSelectionType UpdateMotion(Menu *menu, XEvent *event)
    /* If the selected item is a submenu, show it. */
    ip = GetMenuItem(menu, menu->currentIndex);
    if(ip && IsMenuValid(ip->submenu)) {
-      if(ShowSubmenu(ip->submenu, menu,
-                     menu->x + menu->width - MENU_BORDER_SIZE,
+      if(ShowSubmenu(ip->submenu, menu, runner,
+                     menu->x + menu->width + MENU_BORDER_SIZE,
                      menu->y + menu->offsets[menu->currentIndex])) {
 
          /* Item selected; destroy the menu tree. */
@@ -617,10 +699,10 @@ void DrawMenuItem(Menu *menu, MenuItem *item, int index)
 
    if(!item) {
       if(index == -1 && menu->label) {
-         ResetButton(&button, menu->pixmap, rootGC);
-         button.x = MENU_BORDER_SIZE;
+         ResetButton(&button, menu->pixmap, &rootVisual);
+         button.x = 0;
          button.y = 0;
-         button.width = menu->width - 2 * MENU_BORDER_SIZE - 1;
+         button.width = menu->width - 1;
          button.height = menu->itemHeight - 1;
          button.font = FONT_MENU;
          button.type = BUTTON_LABEL;
@@ -634,7 +716,7 @@ void DrawMenuItem(Menu *menu, MenuItem *item, int index)
    if(item->type != MENU_ITEM_SEPARATOR) {
       ColorType fg;
 
-      ResetButton(&button, menu->pixmap, rootGC);
+      ResetButton(&button, menu->pixmap, &rootVisual);
       if(menu->currentIndex == index) {
          button.type = BUTTON_MENU_ACTIVE;
          fg = COLOR_MENU_ACTIVE_FG;
@@ -643,11 +725,11 @@ void DrawMenuItem(Menu *menu, MenuItem *item, int index)
          fg = COLOR_MENU_FG;
       }
 
-      button.x = MENU_BORDER_SIZE;
+      button.x = 0;
       button.y = menu->offsets[index];
       button.font = FONT_MENU;
-      button.width = menu->width - 2 * MENU_BORDER_SIZE - 1;
-      button.height = menu->itemHeight - 1;
+      button.width = menu->width;
+      button.height = menu->itemHeight;
       button.text = item->name;
       button.icon = item->icon;
       DrawButton(&button);
@@ -671,16 +753,10 @@ void DrawMenuItem(Menu *menu, MenuItem *item, int index)
       }
 
    } else {
-
-      JXSetForeground(display, rootGC, colors[COLOR_MENU_DOWN]);
+      JXSetForeground(display, rootGC, colors[COLOR_MENU_ACTIVE_FG]);
       JXDrawLine(display, menu->pixmap, rootGC, 4,
                  menu->offsets[index] + 2, menu->width - 6,
                  menu->offsets[index] + 2);
-      JXSetForeground(display, rootGC, colors[COLOR_MENU_UP]);
-      JXDrawLine(display, menu->pixmap, rootGC, 4,
-                 menu->offsets[index] + 3, menu->width - 6,
-                 menu->offsets[index] + 3);
-
    }
 
 }
@@ -763,16 +839,15 @@ MenuItem *GetMenuItem(Menu *menu, int index)
 /** Set the active menu item. */
 void SetPosition(Menu *tp, int index)
 {
-
    int y = tp->offsets[index] + tp->itemHeight / 2;
-   if(tp->height > rootHeight) {
+   if(tp->height > tp->screen->height) {
 
       int updated = 0;
       while(y + tp->y < tp->itemHeight / 2) {
          tp->y += tp->itemHeight;
          updated = tp->itemHeight;
       }
-      while(y + tp->y >= rootHeight) {
+      while(y + tp->y >= tp->screen->y + tp->screen->height) {
          tp->y -= tp->itemHeight;
          updated = -tp->itemHeight;
       }
