@@ -16,13 +16,11 @@
 #include "cursor.h"
 #include "desktop.h"
 #include "event.h"
-#include "main.h"
 #include "tray.h"
 #include "timing.h"
 #include "popup.h"
 #include "font.h"
 #include "settings.h"
-#include "event.h"
 
 /** Structure to represent a pager tray component. */
 typedef struct PagerType {
@@ -63,10 +61,12 @@ static void ProcessPagerMotionEvent(TrayComponentType *cp,
 static void StartPagerMove(TrayComponentType *cp, int x, int y);
 
 static void StopPagerMove(ClientNode *np,
-                          int x, int y, int desktop, int hmax, int vmax);
+                          int x, int y, int desktop, MaxFlags maxFlags);
 
 static void PagerMoveController(int wasDestroyed);
 
+static void DrawPager(const PagerType *pp);
+
 static void DrawPagerClient(const PagerType *pp, const ClientNode *np);
 
 static void SignalPager(const TimeType *now, int x, int y, Window w,
@@ -79,7 +79,6 @@ void ShutdownPager(void)
    PagerType *pp;
    for(pp = pagers; pp; pp = pp->next) {
       JXFreePixmap(display, pp->buffer);
-      UnregisterCallback(SignalPager, pp);
    }
 }
 
@@ -88,6 +87,7 @@ void DestroyPager(void)
 {
    PagerType *pp;
    while(pagers) {
+      UnregisterCallback(SignalPager, pagers);
       pp = pagers->next;
       Release(pagers);
       pagers = pp;
@@ -109,6 +109,7 @@ TrayComponentType *CreatePager(char labeled)
    pp->mousey = -settings.doubleClickDelta;
    pp->mouseTime.seconds = 0;
    pp->mouseTime.ms = 0;
+   pp->buffer = None;
 
    cp = CreateTrayComponent();
    cp->object = pp;
@@ -139,7 +140,7 @@ void Create(TrayComponentType *cp)
    Assert(cp->height > 0);
 
    cp->pixmap = JXCreatePixmap(display, rootWindow, cp->width,
-                               cp->height, rootDepth);
+                               cp->height, rootVisual.depth);
    pp->buffer = cp->pixmap;
 
 }
@@ -148,13 +149,7 @@ void Create(TrayComponentType *cp)
 void SetSize(TrayComponentType *cp, int width, int height)
 {
 
-   PagerType *pp;
-
-   Assert(cp);
-
-   pp = (PagerType*)cp->object;
-
-   Assert(pp);
+   PagerType *pp = (PagerType*)cp->object;
 
    if(width) {
 
@@ -182,6 +177,14 @@ void SetSize(TrayComponentType *cp, int width, int height)
       Assert(0);
    }
 
+   if(pp->buffer != None) {
+      JXFreePixmap(display, pp->buffer);
+      pp->buffer = JXCreatePixmap(display, rootWindow, cp->width,
+                                  cp->height, rootVisual.depth);
+      cp->pixmap = pp->buffer;
+      DrawPager(pp);
+   }
+
    pp->scalex = ((pp->deskWidth - 2) << 16) / rootWidth;
    pp->scaley = ((pp->deskHeight - 2) << 16) / rootHeight;
 
@@ -265,7 +268,7 @@ void StartPagerMove(TrayComponentType *cp, int x, int y)
    int oldx, oldy;
    int oldDesk;
    int startx, starty;
-   char hmax, vmax;
+   MaxFlags maxFlags;
 
    pp = (PagerType*)cp->object;
 
@@ -353,16 +356,9 @@ ClientFound:
    }
 
    /* If the client is maximized, unmaximize it. */
-   hmax = 0;
-   if(np->state.status & STAT_HMAX) {
-      hmax = 1;
-   }
-   vmax = 0;
-   if(np->state.status & STAT_VMAX) {
-      vmax = 1;
-   }
-   if(hmax || vmax) {
-      MaximizeClient(np, 0, 0);
+   maxFlags = np->state.maxFlags;
+   if(np->state.maxFlags) {
+      MaximizeClient(np, MAX_NONE);
    }
 
    GetBorderSize(&np->state, &north, &south, &east, &west);
@@ -381,7 +377,7 @@ ClientFound:
    starty = y;
 
    if(!(GetMouseMask() & Button3Mask)) {
-      StopPagerMove(np, oldx, oldy, oldDesk, hmax, vmax);
+      StopPagerMove(np, oldx, oldy, oldDesk, maxFlags);
    }
 
    for(;;) {
@@ -398,7 +394,7 @@ ClientFound:
 
          /* Done when the 3rd mouse button is released. */
          if(event.xbutton.button == Button3) {
-            StopPagerMove(np, oldx, oldy, oldDesk, hmax, vmax);
+            StopPagerMove(np, oldx, oldy, oldDesk, maxFlags);
             return;
          }
          break;
@@ -447,7 +443,7 @@ ClientFound:
          np->y = oldy;
          JXMoveWindow(display, np->parent, np->x - west, np->y - north);
          SendConfigureEvent(np);
-         UpdatePager();
+         RequirePagerUpdate();
 
          break;
 
@@ -461,7 +457,7 @@ ClientFound:
 
 /** Stop an active pager move. */
 void StopPagerMove(ClientNode *np,
-                   int x, int y, int desktop, int hmax, int vmax)
+                   int x, int y, int desktop, MaxFlags maxFlags)
 {
 
    int north, south, east, west;
@@ -480,12 +476,12 @@ void StopPagerMove(ClientNode *np,
    SendConfigureEvent(np);
 
    /* Restore the maximized state of the client. */
-   if(hmax || vmax) {
-      MaximizeClient(np, hmax, vmax);
+   if(maxFlags != MAX_NONE) {
+      MaximizeClient(np, maxFlags);
    }
 
    /* Redraw the pager. */
-   UpdatePager();
+   RequirePagerUpdate();
 
 }
 
@@ -499,11 +495,9 @@ void PagerMoveController(int wasDestroyed)
 
 }
 
-/** Update the pager. */
-void UpdatePager(void)
+/** Draw a pager. */
+void DrawPager(const PagerType *pp)
 {
-
-   PagerType *pp;
    ClientNode *np;
    Pixmap buffer;
    int width, height;
@@ -514,68 +508,79 @@ void UpdatePager(void)
    int textWidth, textHeight;
    int dx, dy;
 
+   buffer = pp->cp->pixmap;
+   width = pp->cp->width;
+   height = pp->cp->height;
+   deskWidth = pp->deskWidth;
+   deskHeight = pp->deskHeight;
+
+   /* Draw the background. */
+   JXSetForeground(display, rootGC, colors[COLOR_PAGER_BG]);
+   JXFillRectangle(display, buffer, rootGC, 0, 0, width, height);
+
+   /* Highlight the current desktop. */
+   JXSetForeground(display, rootGC, colors[COLOR_PAGER_ACTIVE_BG]);
+   dx = currentDesktop % settings.desktopWidth;
+   dy = currentDesktop / settings.desktopWidth;
+   JXFillRectangle(display, buffer, rootGC,
+                   dx * (deskWidth + 1), dy * (deskHeight + 1),
+                   deskWidth, deskHeight);
+
+   /* Draw the labels. */
+   if(pp->labeled) {
+      textHeight = GetStringHeight(FONT_PAGER);
+      if(textHeight < deskHeight) {
+         for(x = 0; x < settings.desktopCount; x++) {
+            dx = x % settings.desktopWidth;
+            dy = x / settings.desktopWidth;
+            name = GetDesktopName(x);
+            textWidth = GetStringWidth(FONT_PAGER, name);
+            if(textWidth < deskWidth) {
+               xc = dx * (deskWidth + 1) + (deskWidth - textWidth) / 2;
+               yc = dy * (deskHeight + 1) + (deskHeight - textHeight) / 2;
+               RenderString(&rootVisual, buffer, FONT_PAGER,
+                            COLOR_PAGER_TEXT, xc, yc, deskWidth, name);
+            }
+         }
+      }
+   }
+
+   /* Draw the clients. */
+   for(x = FIRST_LAYER; x <= LAST_LAYER; x++) {
+      for(np = nodeTail[x]; np; np = np->prev) {
+         DrawPagerClient(pp, np);
+      }
+   }
+
+   /* Draw the desktop dividers. */
+   JXSetForeground(display, rootGC, colors[COLOR_PAGER_FG]);
+   for(x = 1; x < settings.desktopHeight; x++) {
+      JXDrawLine(display, buffer, rootGC,
+                 0, (deskHeight + 1) * x - 1,
+                 width, (deskHeight + 1) * x - 1);
+   }
+   for(x = 1; x < settings.desktopWidth; x++) {
+      JXDrawLine(display, buffer, rootGC,
+                 (deskWidth + 1) * x - 1, 0,
+                 (deskWidth + 1) * x - 1, height);
+   }
+
+}
+
+/** Update the pager. */
+void UpdatePager(void)
+{
+
+   PagerType *pp;
+
    if(JUNLIKELY(shouldExit)) {
       return;
    }
 
    for(pp = pagers; pp; pp = pp->next) {
 
-      buffer = pp->cp->pixmap;
-      width = pp->cp->width;
-      height = pp->cp->height;
-      deskWidth = pp->deskWidth;
-      deskHeight = pp->deskHeight;
-
-      /* Draw the background. */
-      JXSetForeground(display, rootGC, colors[COLOR_PAGER_BG]);
-      JXFillRectangle(display, buffer, rootGC, 0, 0, width, height);
-
-      /* Highlight the current desktop. */
-      JXSetForeground(display, rootGC, colors[COLOR_PAGER_ACTIVE_BG]);
-      dx = currentDesktop % settings.desktopWidth;
-      dy = currentDesktop / settings.desktopWidth;
-      JXFillRectangle(display, buffer, rootGC,
-                      dx * (deskWidth + 1), dy * (deskHeight + 1),
-                      deskWidth, deskHeight);
-
-      /* Draw the labels. */
-      if(pp->labeled) {
-         textHeight = GetStringHeight(FONT_PAGER);
-         if(textHeight < deskHeight) {
-            for(x = 0; x < settings.desktopCount; x++) {
-               dx = x % settings.desktopWidth;
-               dy = x / settings.desktopWidth;
-               name = GetDesktopName(x);
-               textWidth = GetStringWidth(FONT_PAGER, name);
-               if(textWidth < deskWidth) {
-                  xc = dx * (deskWidth + 1) + (deskWidth - textWidth) / 2;
-                  yc = dy * (deskHeight + 1) + (deskHeight - textHeight) / 2;
-                  RenderString(buffer, FONT_PAGER, COLOR_PAGER_TEXT, xc, yc,
-                               deskWidth, name);
-               }
-            }
-         }
-      }
-
-      /* Draw the clients. */
-      for(x = FIRST_LAYER; x <= LAST_LAYER; x++) {
-         for(np = nodeTail[x]; np; np = np->prev) {
-            DrawPagerClient(pp, np);
-         }
-      }
-
-      /* Draw the desktop dividers. */
-      JXSetForeground(display, rootGC, colors[COLOR_PAGER_FG]);
-      for(x = 1; x < settings.desktopHeight; x++) {
-         JXDrawLine(display, buffer, rootGC,
-                    0, (deskHeight + 1) * x - 1,
-                    width, (deskHeight + 1) * x - 1);
-      }
-      for(x = 1; x < settings.desktopWidth; x++) {
-         JXDrawLine(display, buffer, rootGC,
-                    (deskWidth + 1) * x - 1, 0,
-                    (deskWidth + 1) * x - 1, height);
-      }
+      /* Draw the pager. */
+      DrawPager(pp);
 
       /* Tell the tray to redraw. */
       UpdateSpecificTray(pp->cp->tray, pp->cp);
