@@ -11,9 +11,7 @@
 #include "place.h"
 #include "client.h"
 #include "screen.h"
-#include "border.h"
 #include "tray.h"
-#include "main.h"
 #include "settings.h"
 #include "clientlist.h"
 #include "misc.h"
@@ -348,18 +346,20 @@ void SubtractTrayBounds(const TrayType *tp, BoundingBox *box,
    BoundingBox last;
    for(; tp; tp = tp->next) {
 
-      if(tp->layer > layer && !tp->autoHide) {
+      if(tp->layer > layer && tp->autoHide == THIDE_OFF) {
 
          src.x = tp->x;
-         if(src.x < 0) {
-            src.x = rootWidth - src.x;
-         }
          src.y = tp->y;
+         src.width = tp->width + 2 * TRAY_BORDER_SIZE;
+         src.height = tp->height + 2 * TRAY_BORDER_SIZE;
+         if(src.x < 0) {
+            src.width += src.x;
+            src.x = 0;
+         }
          if(src.y < 0) {
-            src.y = rootHeight - src.y;
+            src.height += src.y;
+            src.y = 0;
          }
-         src.width = tp->width;
-         src.height = tp->height;
 
          last = *box;
          SubtractBounds(&src, box);
@@ -758,7 +758,7 @@ void ConstrainPosition(ClientNode *np)
 }
 
 /** Place a maximized client on the screen. */
-void PlaceMaximizedClient(ClientNode *np, char horiz, char vert)
+void PlaceMaximizedClient(ClientNode *np, MaxFlags flags)
 {
 
    BoundingBox box;
@@ -769,18 +769,24 @@ void PlaceMaximizedClient(ClientNode *np, char horiz, char vert)
    np->oldy = np->y;
    np->oldWidth = np->width;
    np->oldHeight = np->height;
+   np->state.maxFlags = flags;
 
    GetBorderSize(&np->state, &north, &south, &east, &west);
 
    sp = GetCurrentScreen(np->x + (east + west + np->width) / 2,
                          np->y + (north + south + np->height) / 2);
    GetScreenBounds(sp, &box);
+   if(!(flags & (MAX_HORIZ | MAX_LEFT | MAX_RIGHT))) {
+      box.x = np->x - west;
+      box.width = np->width + east + west;
+   }
+   if(!(flags & (MAX_VERT | MAX_TOP | MAX_BOTTOM))) {
+      box.y = np->y - north;
+      box.height = np->height + north + south;
+   }
    SubtractTrayBounds(GetTrays(), &box, np->state.layer);
    SubtractStrutBounds(&box, np);
 
-   box.y += north;
-   box.height -= north;
-
    if(box.width > np->maxWidth) {
       box.width = np->maxWidth;
    }
@@ -797,24 +803,60 @@ void PlaceMaximizedClient(ClientNode *np, char horiz, char vert)
       }
    }
 
+   /* Remove window outlines. */
+   if(flags & (MAX_VERT | MAX_TOP)) {
+      north = Max(0, north - 1);
+   }
+   if(flags & (MAX_VERT | MAX_BOTTOM)) {
+      south = Max(0, south - 1);
+   }
+   if(flags & (MAX_HORIZ | MAX_LEFT)) {
+      west = Max(0, west - 1);
+   }
+   if(flags & (MAX_HORIZ | MAX_RIGHT)) {
+      east = Max(0, east - 1);
+   }
+
    /* If maximizing horizontally, update width. */
-   if(horiz) {
+   if(flags & MAX_HORIZ) {
       np->x = box.x;
       np->width = box.width;
       if(!(np->state.status & STAT_IIGNORE)) {
          np->width -= ((box.width - np->baseWidth) % np->xinc);
       }
-      np->state.status |= STAT_HMAX;
+   } else if(flags & MAX_LEFT) {
+      np->x = box.x;
+      np->width = box.width / 2 - west;
+      if(!(np->state.status & STAT_IIGNORE)) {
+         np->width -= ((box.width - np->baseWidth) % np->xinc);
+      }
+   } else if(flags & MAX_RIGHT) {
+      np->x = box.x + box.width / 2 + west;
+      np->width = box.width / 2 - east;
+      if(!(np->state.status & STAT_IIGNORE)) {
+         np->width -= ((box.width - np->baseWidth) % np->xinc);
+      }
    }
 
    /* If maximizing vertically, update height. */
-   if(vert) {
-      np->y = box.y;
-      np->height = box.height;
+   if(flags & MAX_VERT) {
+      np->y = box.y + north;
+      np->height = box.height - north;
+      if(!(np->state.status & STAT_IIGNORE)) {
+         np->height -= ((box.height - np->baseHeight) % np->yinc);
+      }
+   } else if(flags & MAX_TOP) {
+      np->y = box.y + north;
+      np->height = box.height / 2 - north - south;
+      if(!(np->state.status & STAT_IIGNORE)) {
+         np->height -= ((box.height - np->baseHeight) % np->yinc);
+      }
+   } else if(flags & MAX_BOTTOM) {
+      np->y = box.y + box.height / 2 + north;
+      np->height = box.height / 2 - north - south;
       if(!(np->state.status & STAT_IIGNORE)) {
          np->height -= ((box.height - np->baseHeight) % np->yinc);
       }
-      np->state.status |= STAT_VMAX;
    }
 
 }
