@@ -11,14 +11,12 @@
 #include "resize.h"
 #include "client.h"
 #include "outline.h"
-#include "main.h"
 #include "cursor.h"
 #include "misc.h"
 #include "pager.h"
 #include "status.h"
 #include "key.h"
 #include "event.h"
-#include "border.h"
 #include "settings.h"
 
 static char shouldStopResize;
@@ -58,7 +56,7 @@ void ResizeClient(ClientNode *np, BorderActionType action,
    if(!(np->state.border & BORDER_RESIZE)) {
       return;
    }
-   if(np->state.status & (STAT_VMAX | STAT_HMAX | STAT_FULLSCREEN)) {
+   if((np->state.status & STAT_FULLSCREEN) || np->state.maxFlags) {
       return;
    }
 
@@ -205,7 +203,7 @@ void ResizeClient(ClientNode *np, BorderActionType action,
                SendConfigureEvent(np);
             }
 
-            UpdatePager();
+            RequirePagerUpdate();
 
          }
 
@@ -233,7 +231,7 @@ void ResizeClientKeyboard(ClientNode *np)
    if(!(np->state.border & BORDER_RESIZE)) {
       return;
    }
-   if(np->state.status & (STAT_VMAX | STAT_HMAX | STAT_FULLSCREEN)) {
+   if((np->state.status & STAT_FULLSCREEN) || np->state.maxFlags) {
       return;
    }
 
@@ -369,7 +367,7 @@ void ResizeClientKeyboard(ClientNode *np)
             SendConfigureEvent(np);
          }
 
-         UpdatePager();
+         RequirePagerUpdate();
 
       }
 
@@ -385,11 +383,11 @@ void StopResize(ClientNode *np)
 
    /* Set the old width/height if maximized so the window
     * is restored to the new size. */
-   if(np->state.status & STAT_VMAX) {
+   if(np->state.maxFlags & MAX_VERT) {
       np->oldWidth = np->width;
       np->oldx = np->x;
    }
-   if(np->state.status & STAT_HMAX) {
+   if(np->state.maxFlags & MAX_HORIZ) {
       np->oldHeight = np->height;
       np->oldy = np->y;
    }
