@@ -6,7 +6,6 @@
 
 #include "jwm.h"
 #include "settings.h"
-#include "debug.h"
 #include "misc.h"
 
 Settings settings;
@@ -34,19 +33,24 @@ void InitializeSettings(void)
    settings.popupEnabled = 1;
    settings.activeClientOpacity = UINT_MAX;
    settings.inactiveClientOpacity = (unsigned int)(0.75 * UINT_MAX);
-   settings.borderWidth = 4;
-   settings.titleHeight = 20;
+   settings.borderWidth = 5;
+   settings.titleHeight = 22;
    settings.desktopWidth = 4;
    settings.desktopHeight = 1;
    settings.menuOpacity = UINT_MAX;
    settings.taskInsertMode = INSERT_RIGHT;
    settings.exitConfirmation = 1;
+   settings.cornerRadius = 4;
+   settings.handles = 0;
+   settings.groupTasks = 0;
 }
 
 /** Make sure settings are reasonable. */
 void StartupSettings(void)
 {
 
+   FixRange(&settings.cornerRadius, 0, 5, 4);
+
    FixRange(&settings.borderWidth, 1, 128, 4);
    FixRange(&settings.titleHeight, 2, 256, 20);
 
@@ -81,4 +85,3 @@ void FixRange(unsigned int *value,
       *value = def_value;
    }
 }
-
