@@ -20,14 +20,13 @@
 #include "screen.h"
 #include "settings.h"
 #include "event.h"
-#include "gradient.h"
 #include "client.h"
+#include "misc.h"
 
 #define DEFAULT_TRAY_WIDTH 32
 #define DEFAULT_TRAY_HEIGHT 32
 
 static TrayType *trays;
-static Window supportingWindow;
 static unsigned int trayCount;
 
 static void HandleTrayExpose(TrayType *tp, const XExposeEvent *event);
@@ -57,7 +56,6 @@ void InitializeTray(void)
 {
    trays = NULL;
    trayCount = 0;
-   supportingWindow = None;
 }
 
 /** Startup trays. */
@@ -99,10 +97,16 @@ void StartupTray(void)
       attrMask |= CWBackPixel;
       attr.background_pixel = colors[COLOR_TRAY_BG2];
 
+      attrMask |= CWBorderPixel;
+      attr.border_pixel = colors[COLOR_TRAY_OUTLINE];
+
+      Assert(tp->width > 0);
+      Assert(tp->height > 0);
       tp->window = JXCreateWindow(display, rootWindow,
                                   tp->x, tp->y, tp->width, tp->height,
-                                  0, rootDepth, InputOutput, rootVisual,
-                                  attrMask, &attr);
+                                  TRAY_BORDER_SIZE,
+                                  rootVisual.depth, InputOutput,
+                                  rootVisual.visual, attrMask, &attr);
 
       if(settings.trayOpacity < UINT_MAX) {
          /* Can't use atoms yet as it hasn't been initialized. */
@@ -115,13 +119,13 @@ void StartupTray(void)
       SetDefaultCursor(tp->window);
 
       /* Create and layout items on the tray. */
-      xoffset = tp->border;
-      yoffset = tp->border;
+      xoffset = 0;
+      yoffset = 0;
       for(cp = tp->components; cp; cp = cp->next) {
 
          if(cp->Create) {
             if(tp->layout == LAYOUT_HORIZONTAL) {
-               height = tp->height - 2 * tp->border;
+               height = tp->height;
                width = cp->width;
                if(width == 0) {
                   width = variableSize;
@@ -131,7 +135,7 @@ void StartupTray(void)
                   }
                }
             } else {
-               width = tp->width - 2 * tp->border;
+               width = tp->width;
                height = cp->height;
                if(height == 0) {
                   height = variableSize;
@@ -141,8 +145,8 @@ void StartupTray(void)
                   }
                }
             }
-            cp->width = width;
-            cp->height = height;
+            cp->width = Max(1, width);
+            cp->height = Max(1, height);
             (cp->Create)(cp);
          }
 
@@ -170,8 +174,8 @@ void StartupTray(void)
 
    }
 
-   UpdatePager();
-   UpdateTaskBar();
+   RequirePagerUpdate();
+   RequireTaskUpdate();
 
 }
 
@@ -189,12 +193,6 @@ void ShutdownTray(void)
          }
       }
       JXDestroyWindow(display, tp->window);
-      UnregisterCallback(SignalTray, tp);
-   }
-
-   if(supportingWindow != None) {
-      XDestroyWindow(display, supportingWindow);
-      supportingWindow = None;
    }
 
 }
@@ -208,7 +206,7 @@ void DestroyTray(void)
 
    while(trays) {
       tp = trays->next;
-
+      UnregisterCallback(SignalTray, trays);
       while(trays->components) {
          cp = trays->components->next;
          Release(trays->components);
@@ -229,19 +227,20 @@ TrayType *CreateTray(void)
 
    tp = Allocate(sizeof(TrayType));
 
+   tp->requestedX = 0;
+   tp->requestedY = -1;
    tp->x = 0;
    tp->y = -1;
    tp->requestedWidth = 0;
    tp->requestedHeight = 0;
    tp->width = 0;
    tp->height = 0;
-   tp->border = 1;
    tp->layer = DEFAULT_TRAY_LAYER;
    tp->layout = LAYOUT_HORIZONTAL;
    tp->valign = TALIGN_FIXED;
    tp->halign = TALIGN_FIXED;
 
-   tp->autoHide = 0;
+   tp->autoHide = THIDE_OFF;
    tp->hidden = 0;
 
    tp->window = None;
@@ -289,6 +288,7 @@ TrayComponentType *CreateTrayComponent(void)
    cp->ProcessButtonPress = NULL;
    cp->ProcessButtonRelease = NULL;
    cp->ProcessMotionEvent = NULL;
+   cp->Redraw = NULL;
 
    cp->next = NULL;
 
@@ -327,7 +327,6 @@ int ComputeMaxWidth(TrayType *tp)
    for(cp = tp->components; cp; cp = cp->next) {
       temp = cp->width;
       if(temp > 0) {
-         temp += 2 * tp->border;
          if(temp > result) {
             result = temp;
          }
@@ -345,7 +344,7 @@ int ComputeTotalWidth(TrayType *tp)
    TrayComponentType *cp;
    int result;
 
-   result = 2 * tp->border;
+   result = 0;
    for(cp = tp->components; cp; cp = cp->next) {
       result += cp->width;
    }
@@ -366,7 +365,6 @@ int ComputeMaxHeight(TrayType *tp)
    for(cp = tp->components; cp; cp = cp->next) {
       temp = cp->height;
       if(temp > 0) {
-         temp += 2 * tp->border;
          if(temp > result) {
             result = temp;
          }
@@ -384,7 +382,7 @@ int ComputeTotalHeight(TrayType *tp)
    TrayComponentType *cp;
    int result;
 
-   result = 2 * tp->border;
+   result = 0;
    for(cp = tp->components; cp; cp = cp->next) {
       result += cp->height;
    }
@@ -439,7 +437,6 @@ void ComputeTraySize(TrayType *tp)
       if(tp->height == 0) {
          tp->height = ComputeMaxHeight(tp);
       }
-
       if(tp->height == 0) {
          tp->height = DEFAULT_TRAY_HEIGHT;
       }
@@ -449,7 +446,6 @@ void ComputeTraySize(TrayType *tp)
       if(tp->width == 0) {
          tp->width = ComputeMaxWidth(tp);
       }
-
       if(tp->width == 0) {
          tp->width = DEFAULT_TRAY_WIDTH;
       }
@@ -460,13 +456,17 @@ void ComputeTraySize(TrayType *tp)
    for(cp = tp->components; cp; cp = cp->next) {
       if(cp->SetSize) {
          if(tp->layout == LAYOUT_HORIZONTAL) {
-            (cp->SetSize)(cp, 0, tp->height - 2 * tp->border);
+            (cp->SetSize)(cp, 0, tp->height);
          } else {
-            (cp->SetSize)(cp, tp->width - 2 * tp->border, 0);
+            (cp->SetSize)(cp, tp->width, 0);
          }
       }
    }
 
+   /* Initialize the coordinates. */
+   tp->x = tp->requestedX;
+   tp->y = tp->requestedY;
+
    /* Determine on which screen the tray will reside. */
    switch(tp->valign) {
    case TALIGN_TOP:
@@ -504,9 +504,9 @@ void ComputeTraySize(TrayType *tp)
       }
       break;
    }
+   sp = GetCurrentScreen(x, y);
 
    /* Determine the missing dimension. */
-   sp = GetCurrentScreen(x, y);
    if(tp->layout == LAYOUT_HORIZONTAL) {
       if(tp->width == 0) {
          if(CheckHorizontalFill(tp)) {
@@ -534,34 +534,38 @@ void ComputeTraySize(TrayType *tp)
    /* Compute the tray location. */
    switch(tp->valign) {
    case TALIGN_TOP:
-      tp->y = sp->y;
+      tp->y = sp->y - TRAY_BORDER_SIZE;
       break;
    case TALIGN_BOTTOM:
-      tp->y = sp->y + sp->height - tp->height;
+      tp->y = sp->y + sp->height - tp->height + TRAY_BORDER_SIZE;
       break;
    case TALIGN_CENTER:
       tp->y = sp->y + (sp->height - tp->height) / 2;
       break;
    default:
       if(tp->y < 0) {
-         tp->y = sp->y + sp->height - tp->height;
+         tp->y = sp->y + sp->height - tp->height + TRAY_BORDER_SIZE;
+      } else {
+         tp->y -= TRAY_BORDER_SIZE;
       }
       break;
    }
 
    switch(tp->halign) {
    case TALIGN_LEFT:
-      tp->x = sp->x;
+      tp->x = sp->x - TRAY_BORDER_SIZE;
       break;
    case TALIGN_RIGHT:
-      tp->x = sp->x + sp->width - tp->width;
+      tp->x = sp->x + sp->width - tp->width + TRAY_BORDER_SIZE;
       break;
    case TALIGN_CENTER:
       tp->x = sp->x + (sp->width - tp->width) / 2;
       break;
    default:
       if(tp->x < 0) {
-         tp->x = sp->x + sp->width - tp->width;
+         tp->x = sp->x + sp->width - tp->width + TRAY_BORDER_SIZE;
+      } else {
+         tp->x -= TRAY_BORDER_SIZE;
       }
       break;
    }
@@ -613,34 +617,40 @@ void HideTray(TrayType *tp)
    const ScreenType *sp;
    int x, y;
 
+   /* Don't hide if the tray is raised. */
+   if(tp->autoHide & THIDE_RAISED) {
+      return;
+   }
+
    tp->hidden = 1;
 
    /* Determine where to move the tray. */
    sp = GetCurrentScreen(tp->x, tp->y);
-   if(tp->layout == LAYOUT_HORIZONTAL) {
-
-      x = tp->x;
-
-      if(tp->y >= sp->y + (sp->height / 2)) {
-         y = sp->y + sp->height - 1;
-      } else {
-         y = sp->y - tp->height + 1;
-      }
-
-   } else {
-
+   switch(tp->autoHide) {
+   case THIDE_LEFT:
+      x = sp->y - tp->width - TRAY_BORDER_SIZE;
       y = tp->y;
-
-      if(tp->x >= sp->x + (sp->width / 2)) {
-         x = sp->x + sp->width - 1;
-      } else {
-         x = sp->x - tp->width + 1;
-      }
-
+      break;
+   case THIDE_RIGHT:
+      x = sp->y + sp->width - TRAY_BORDER_SIZE;
+      y = tp->y;
+      break;
+   case THIDE_TOP:
+      x = tp->x;
+      y = sp->y - tp->height - TRAY_BORDER_SIZE;
+      break;
+   case THIDE_BOTTOM:
+      x = tp->x;
+      y = sp->y + sp->height - TRAY_BORDER_SIZE;
+      break;
+   default:
+      Assert(0);
+      break;
    }
 
-   /* Move it. */
+   /* Move and redraw. */
    JXMoveWindow(display, tp->window, x, y);
+   DrawSpecificTray(tp);
 
 }
 
@@ -682,7 +692,7 @@ char ProcessTrayEvent(const XEvent *event)
 void SignalTray(const TimeType *now, int x, int y, Window w, void *data)
 {
    TrayType *tp = (TrayType*)data;
-   if(tp->autoHide == 1 && !tp->hidden && !menuShown) {
+   if(tp->autoHide != THIDE_OFF && !tp->hidden && !menuShown) {
       if(x < tp->x || x >= tp->x + tp->width
          || y < tp->y || y >= tp->y + tp->height) {
          HideTray(tp);
@@ -708,22 +718,23 @@ TrayComponentType *GetTrayComponent(TrayType *tp, int x, int y)
 
    TrayComponentType *cp;
    int xoffset, yoffset;
-   int width, height;
 
-   xoffset = tp->border;
-   yoffset = tp->border;
+   xoffset = 0;
+   yoffset = 0;
    for(cp = tp->components; cp; cp = cp->next) {
-      width = cp->width;
-      height = cp->height;
-      if(x >= xoffset && x < xoffset + width) {
-         if(y >= yoffset && y < yoffset + height) {
+      int startx = xoffset;
+      int starty = yoffset;
+      int width = cp->width;
+      int height = cp->height;
+      if(x >= startx && x < startx + width) {
+         if(y >= starty && y < starty + height) {
             return cp;
          }
       }
       if(tp->layout == LAYOUT_HORIZONTAL) {
-         xoffset += width;
+         xoffset += cp->width;
       } else {
-         yoffset += height;
+         yoffset += cp->height;
       }
    }
 
@@ -807,40 +818,11 @@ void DrawSpecificTray(const TrayType *tp)
 {
 
    TrayComponentType *cp;
-   int x;
 
-   Assert(tp);
-
-   /* Draw components. */
    for(cp = tp->components; cp; cp = cp->next) {
       UpdateSpecificTray(tp, cp);
    }
 
-   /* Draw the border. */
-   for(x = 0; x < tp->border; x++) {
-
-      /* Top */
-      JXSetForeground(display, rootGC, colors[COLOR_TRAY_UP]);
-      JXDrawLine(display, tp->window, rootGC, 0, x,
-                 tp->width - x - 1, x);
-
-      /* Bottom */
-      JXSetForeground(display, rootGC, colors[COLOR_TRAY_DOWN]);
-      JXDrawLine(display, tp->window, rootGC, x + 1, tp->height - x - 1,
-                 tp->width - x - 2, tp->height - x - 1);
-
-      /* Left */
-      JXSetForeground(display, rootGC, colors[COLOR_TRAY_UP]);
-      JXDrawLine(display, tp->window, rootGC, x, x,
-                 x, tp->height - x - 1);
-
-      /* Right */
-      JXSetForeground(display, rootGC, colors[COLOR_TRAY_DOWN]);
-      JXDrawLine(display, tp->window, rootGC, tp->width - x - 1, x + 1,
-                 tp->width - x - 1, tp->height - x - 1);
-
-   }
-
 }
 
 /** Raise tray windows. */
@@ -848,7 +830,7 @@ void RaiseTrays(void)
 {
    TrayType *tp;
    for(tp = trays; tp; tp = tp->next) {
-      tp->autoHide |= 2;
+      tp->autoHide |= THIDE_RAISED;
       ShowTray(tp);
       JXRaiseWindow(display, tp->window);
    }
@@ -859,16 +841,21 @@ void LowerTrays(void)
 {
    TrayType *tp;
    for(tp = trays; tp; tp = tp->next) {
-      tp->autoHide &= ~2;
+      tp->autoHide &= ~THIDE_RAISED;
    }
-   RestackClients();
+   RequireRestack();
 }
 
 /** Update a specific component on a tray. */
 void UpdateSpecificTray(const TrayType *tp, const TrayComponentType *cp)
 {
 
-   if(cp->pixmap != None && !shouldExit) {
+   if(JUNLIKELY(shouldExit)) {
+      return;
+   }
+
+   /* If the tray is hidden, draw only the background. */
+   if(!tp->hidden && cp->pixmap != None) {
       JXCopyArea(display, cp->pixmap, tp->window, rootGC, 0, 0,
                  cp->width, cp->height, cp->x, cp->y);
    }
@@ -896,8 +883,8 @@ void LayoutTray(TrayType *tp, int *variableSize, int *variableRemainder)
 
    /* Get the remaining size after setting fixed size components. */
    /* Also, keep track of the number of variable size components. */
-   width = tp->width - 2 * tp->border;
-   height = tp->height - 2 * tp->border;
+   width = tp->width;
+   height = tp->height;
    variableCount = 0;
    for(cp = tp->components; cp; cp = cp->next) {
       if(tp->layout == LAYOUT_HORIZONTAL) {
@@ -943,6 +930,9 @@ void LayoutTray(TrayType *tp, int *variableSize, int *variableRemainder)
       }
    }
 
+   tp->width = Max(1, tp->width);
+   tp->height = Max(1, tp->height);
+
 }
 
 /** Resize a tray. */
@@ -960,8 +950,8 @@ void ResizeTray(TrayType *tp)
    LayoutTray(tp, &variableSize, &variableRemainder);
 
    /* Reposition items on the tray. */
-   xoffset = tp->border;
-   yoffset = tp->border;
+   xoffset = 0;
+   yoffset = 0;
    for(cp = tp->components; cp; cp = cp->next) {
 
       cp->x = xoffset;
@@ -971,7 +961,7 @@ void ResizeTray(TrayType *tp)
 
       if(cp->Resize) {
          if(tp->layout == LAYOUT_HORIZONTAL) {
-            height = tp->height - 2 * tp->border;
+            height = tp->height;
             width = cp->width;
             if(width == 0) {
                width = variableSize;
@@ -981,7 +971,7 @@ void ResizeTray(TrayType *tp)
                }
             }
          } else {
-            width = tp->width - 2 * tp->border;
+            width = tp->width;
             height = cp->height;
             if(height == 0) {
                height = variableSize;
@@ -1010,7 +1000,7 @@ void ResizeTray(TrayType *tp)
    JXMoveResizeWindow(display, tp->window, tp->x, tp->y,
                       tp->width, tp->height);
 
-   UpdateTaskBar();
+   RequireTaskUpdate();
    DrawSpecificTray(tp);
 
    if(tp->hidden) {
@@ -1045,27 +1035,11 @@ unsigned int GetTrayCount(void)
    return trayCount;
 }
 
-/** Get a supporting window to use. */
-Window GetSupportingWindow(void)
-{
-
-   if(trays) {
-      return trays->window;
-   } else if(supportingWindow != None) {
-      return supportingWindow;
-   } else {
-      supportingWindow = JXCreateSimpleWindow(display, rootWindow,
-                                              0, 0, 1, 1, 0, 0, 0);
-      return supportingWindow;
-   }
-
-}
-
 /** Determine if a tray should autohide. */
-void SetAutoHideTray(TrayType *tp, char v)
+void SetAutoHideTray(TrayType *tp, TrayAutoHideType autohide)
 {
    Assert(tp);
-   tp->autoHide = v;
+   tp->autoHide = autohide;
 }
 
 /** Set the x-coordinate of a tray. */
@@ -1073,7 +1047,7 @@ void SetTrayX(TrayType *tp, const char *str)
 {
    Assert(tp);
    Assert(str);
-   tp->x = atoi(str);
+   tp->requestedX = atoi(str);
 }
 
 /** Set the y-coordinate of a tray. */
@@ -1081,7 +1055,7 @@ void SetTrayY(TrayType *tp, const char *str)
 {
    Assert(tp);
    Assert(str);
-   tp->y = atoi(str);
+   tp->requestedY = atoi(str);
 }
 
 /** Set the width of a tray. */
@@ -1169,42 +1143,45 @@ void SetTrayLayer(TrayType *tp, WinLayerType layer)
 /** Set the horizontal tray alignment. */
 void SetTrayHorizontalAlignment(TrayType *tp, const char *str)
 {
+   static const StringMappingType mapping[] = {
+      { "center",    TALIGN_CENTER  },
+      { "fixed",     TALIGN_FIXED   },
+      { "left",      TALIGN_LEFT    },
+      { "right",     TALIGN_RIGHT   }
+   };
 
-   Assert(tp);
-
-   if(!str || !strcmp(str, "fixed")) {
+   if(!str) {
       tp->halign = TALIGN_FIXED;
-   } else if(!strcmp(str, "left")) {
-      tp->halign = TALIGN_LEFT;
-   } else if(!strcmp(str, "right")) {
-      tp->halign = TALIGN_RIGHT;
-   } else if(!strcmp(str, "center")) {
-      tp->halign = TALIGN_CENTER;
    } else {
-      Warning(_("invalid tray horizontal alignment: \"%s\""), str);
-      tp->halign = TALIGN_FIXED;
+      const int x = FindValue(mapping, ARRAY_LENGTH(mapping), str);
+      if(JLIKELY(x >= 0)) {
+         tp->halign = x;
+      } else {
+         Warning(_("invalid tray horizontal alignment: \"%s\""), str);
+         tp->halign = TALIGN_FIXED;
+      }
    }
-
 }
 
 /** Set the vertical tray alignment. */
 void SetTrayVerticalAlignment(TrayType *tp, const char *str)
 {
+   static const StringMappingType mapping[] = {
+      { "bottom",    TALIGN_BOTTOM  },
+      { "center",    TALIGN_CENTER  },
+      { "fixed",     TALIGN_FIXED   },
+      { "top",       TALIGN_TOP     }
+   };
 
-   Assert(tp);
-
-   if(!str || !strcmp(str, "fixed")) {
+   if(!str) {
       tp->valign = TALIGN_FIXED;
-   } else if(!strcmp(str, "top")) {
-      tp->valign = TALIGN_TOP;
-   } else if(!strcmp(str, "bottom")) {
-      tp->valign = TALIGN_BOTTOM;
-   } else if(!strcmp(str, "center")) {
-      tp->valign = TALIGN_CENTER;
    } else {
-      Warning(_("invalid tray vertical alignment: \"%s\""), str);
-      tp->valign = TALIGN_FIXED;
+      const int x = FindValue(mapping, ARRAY_LENGTH(mapping), str);
+      if(JLIKELY(x >= 0)) {
+         tp->valign = x;
+      } else {
+         Warning(_("invalid tray vertical alignment: \"%s\""), str);
+         tp->valign = TALIGN_FIXED;
+      }
    }
-
 }
-
